/**
 * @class Sb.auxilaryModel.Image
 * @extends Sb.model.Node
 * Image get/set facade (DOM as source).
 * @constructor
 * @param {Object} config Configuration for image
 */
Sb.auxilaryModel.Image = Ext.extend(Sb.model.Node, {
	constructor: function(config) {
		config = config || {};

		/**
		 * Current image dom element to operate with.
		 *
		 * @type Ext.Element
		 * @property el
		 */
		this.el = {};

		/**
		 * Can we update element and it`s DOM?
		 *
		 * @type Boolean
		 * @property canUpdateEl
		 */
		this.canUpdateEl = true;

		this.addEvents(
			/**
			 * @event elementchange
			 * Fires when the element was set or changed.
			 * @param {Sb.auxilaryModel.Image} this This image.
			 */
			'elementchange',
			/**
			 * @event alignchange
			 * Fires when image align setted or changed.
			 * @param {Sb.auxilaryModel.Image} this This image.
			 */
			'alignchange',
			/**
			 * @event heightchange
			 * Fires when image height setted or changed.
			 * @param {Sb.auxilaryModel.Image} this This image.
			 */
			'heightchange'
		);
		Sb.auxilaryModel.Image.superclass.constructor.call(this, config);
	},

	/**
	 * Sets current image`s DOM element as Ext`s Element.
	 *
	 * @param {Mixed} domEl DOM element link or query
	 */
	setElement: function(domEl) {
		var elCandidate = Ext.get(domEl);
		if (null == elCandidate || !elCandidate.is('img')) {
			return false;
		}
		this.el = elCandidate;
		this.onBeforeElementChange();
		this.fireEvent('elementchange', this);
		return true;
	},

	/**
	 * Fires on changing of single property change to allow us fire up more abstract change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function(key) {
		var value = this.data[key];
		if ('sourceUrl' == key) {
			var rawSourceFileName	= decodeURIComponent(value.split('/').pop());
			rawSourceFileName		= rawSourceFileName.replace(/=/g, '%');
			var sourceFileName		= decodeURIComponent(rawSourceFileName);
			sourceFileName			= sourceFileName.replace('\\', '');
			this.unserialize({
				rawSourceFileName	: rawSourceFileName,
				sourceFileName		: sourceFileName
			});
		} else if ('heightProportion' == key) {
			if (null == value) {
				this.unserialize({
					width	: '',
					height	: ''
				});
			}
		} else if ('width' == key || 'height' == key) {
			var height		= parseInt(this.data.height);
			var width		= parseInt(this.data.width);
			if (height * width) {
				if (null == this.data.heightProportion) {
					this.setProperty('heightProportion', height / width);
				} else if ('width' == key) {
					height = Math.round(width * this.data.heightProportion);
					if (0 == height) {
						height = 1;
					}
					this.setProperty('height', height);
				} else if ('height' == key) {
					width = Math.round(height / this.data.heightProportion);
					if (0 == width) {
						width = 1;
					}
					this.setProperty('width', width);
				}
			}
			value = this.data[key];
		} else if ('alignTo' == key) {
			value = value.toLowerCase();
			if (!(/^(left|right|center)$/).test(value)) {
				value = 'none';
			}
			this.data[key] = value;
		}
		this.updateDomElement(key, value);
		if ('alignTo' == key) {
			this.fireEvent('alignchange', this);
		}
	},

	/**
	 * Updates properties from img to data.
	 */
	onBeforeElementChange: function() {
		var el				= this.el;
		this.canUpdateEl	= false;
		this.unserialize({
			sourceUrl			: el.getAttribute('_mce_src') || el.getAttribute('src'),
			heightProportion	: null,
			width				: el.getWidth(),
			height				: el.getHeight(),
			alignTo				: this.getAlignment(),
			alternativeText		: el.getAttribute('title') || el.getAttribute('alt')
		});
		this.canUpdateEl	= true;
	},

	/**
	 * Updates DOM with values. Also updates editor-specific attributes.
	 *
	 * @param {String} key
	 * @param {String|Number} value
	 */
	updateDomElement: function(key, value) {
		if (!this.canUpdateEl) {
			return;
		}
		var el = this.el;
		if (key == 'sourceUrl') {
			el.set({src: value, _mce_src: value});
		} else if (key == 'width') {
			el.set({width: value});
		} else if (key == 'height') {
			el.set({height: value});
			this.fireEvent('heightchange', this);
		} else if (key == 'alignTo') {
			this.setAlignment(value);
		} else if (key == 'alternativeText') {
			el.set({alt: value});
			el.set({title: value});
		}
	},

	/**
	 * Always store dynamic property heightProportion as null
	 */
	saveToMemento: function() {
		return Ext.apply(
			Sb.auxilaryModel.Image.superclass.saveToMemento.call(this),
			{heightProportion: null}
		);
	},

	/**
	 * Set alignment of selected element
	 *
	 * @param {String} value
	 */
	setAlignment: function(value) {
		var el		= this.el;
		var style	= {
			'float'		: '',
			marginLeft	: '',
			marginRight	: '',
			display		: ''
		};
		if ('left' == value) {
			style['float']		= 'left';
			style.marginRight	= '20px';
		} else if ('right' == value) {
			style['float']		= 'right';
			style.marginLeft	= '20px';
		} else if ('center' == value) {
			style.marginLeft	= 'auto';
			style.marginRight	= 'auto';
			style.display		= 'block';
		}
		el.setStyle(style);
		this.cacheStyleInEditor(el);
	},

	/**
	 * Get alignment of selected element
	 *
	 * @param {String} alignment
	 */
	getAlignment: function() {
		var el		= this.el;
		var cssFloat= el.getStyle('float');
		var value	= 'none';
		if ('left' == cssFloat || 'right' == cssFloat) {
			value	= cssFloat;
		} else if ('block' == el.getStyle('display')) {
			value	= 'center';
		}
		return value;
	},

	/**
	 * Selects image in editor
	 */
	selectInEditor: function() {
		if (tinyMCE.activeEditor && this.el.dom) {
			tinyMCE.activeEditor.selection.select(this.el.dom);
		}
	},

	/**
	 * Deselects image in editor
	 */
	deselectInEditor: function() {
		if (tinyMCE.activeEditor) {
			tinyMCE.activeEditor.selection.collapse();
		}
	},

	/**
	 * Removes image from editor
	 */
	removeFromEditor: function() {
		if (tinyMCE.activeEditor && this.el.dom) {
			tinyMCE.activeEditor.dom.remove(this.el.dom);
			delete this.el.dom;
		}
	},

	/**
	 * Updates editor cached inline style by copying it from current element style
	 *
	 * @param {Ext.Element} el Element on which editor`s cached style value is updated [optional]
	 */
	cacheStyleInEditor: function(el) {
		el			= el || this.el;
		var style	= el.getAttribute('style');
		el.set({
			_mce_style: Ext.isIE ? style.cssText.toString() : style.toString()
		});
	}
});