/**
 * Edit image dialog for text widget
 *
 * @class Sb.editor.ImageEditDialog
 * @extends Sb.Window
 */
Sb.editor.ImageEditDialog = Ext.extend(Sb.Window, {
	layout			: 'column',
	width			: 475,
	title			: SbGetLocaleByKey('imageEditDialogEdit'),
	modal			: true,
	resizable		: false,
	closeAction		: 'cancelHandler',
	cls				: 'sb-dialog-image',

	constructor: function(config) {
		config = Ext.apply({
			/**
			 * @cfg {Mixed} domEl DOM node of edited image.
			 */
			domEl				: null,
			/**
			 * @cfg {Boolean} isUploaded When image is uploaded and not changed
			 */
			isUploaded			: false,
			/**
			 * @cfg {Function} afterHeightChange Fires after height changed
			 */
			afterHeightChange	: Ext.emptyFn
		}, config);

		/**
		 * Current image model (operating with DOM).
		 *
		 * @type Sb.auxilaryModel.Image
		 * @property model
		 */
		this.model		= new Sb.auxilaryModel.Image();

		/**
		 * Memento for reverting on Cancel
		 *
		 * @type Object
		 * @property memento
		 */
		this.memento	= null;

		Sb.editor.ImageEditDialog.superclass.constructor.call(this, config);
	},

	initComponent: function() {
		this.buttons	= [{
			text		: SbGetLocaleByKey('ok'),
			handler		: this.okHandler.createDelegate(this)
		}, {
			text		: SbGetLocaleByKey('cancelButtonTitle'),
			handler		: this.cancelHandler.createDelegate(this)
		}];

		var columnDefault = {
			columnWidth	: 0.5,
			border		: false
		};
		var leftColumn	= new Ext.Container(
			Ext.apply({
				cls			: 'sb-dialog-image-block1'
			}, columnDefault)
		);
		var rightColumn	= new Ext.form.FormPanel(
			Ext.apply({
				cls			: 'sb-dialog-image-block2',
				labelAlign	: 'top'
			}, columnDefault)
		);
		this.items		= [leftColumn, rightColumn];

		// Left column controls
		/**
		 * Preview label, that shows us file name
		 * @type Ext.form.Label
		 * @property previewLbl
		 */
		this.previewLbl	= new Ext.form.Label();
		var previewCtn	= new Ext.Container({
			cls			: 'sb-dialog-image-preview'
		});
		/**
		 * Preview image, that will shows us current image or uploaded image
		 * @type Ext.BoxComponent
		 * @property previewImg
		 */
		this.previewImg = new Ext.BoxComponent({
			autoEl		: {
				tag		: 'IMG',
				src		: 'images/blank.gif'
			}
		});
		previewCtn.add(this.previewImg);

		var sizeInPxLbl	= new Ext.form.Label({
			text		: SbGetLocaleByKey('imageEditDialogSizeInPixels')
		});

		var dimensionDefault = {
			value		: 0,
			maskRe		: /\d+/
		};

		/**
		 * Image width field
		 * @type Ext.form.TextField
		 * @property widthFld
		 */
		this.widthFld	= new Ext.form.TextField(dimensionDefault);
		var xLbl		= new Ext.Component({
			autoEl		: {
				tag		: 'span',
				html	: 'x'
			}
		});
		/**
		 * Image height field
		 * @type Ext.form.TextField
		 * @property heightFld
		 */
		this.heightFld	= new Ext.form.TextField(dimensionDefault);

		var uploadFrm	= new Ext.form.FormPanel({
			fileUpload	: true,
			border		: false,
			hideLabels	: true
		});
		/**
		 * Change image button
		 * @type Ext.ux.form.FileUploadField
		 * @property uploadFld
		 */
		this.uploadFld	= new Ext.ux.form.FileUploadField({
			regex		: new RegExp('\\.(' + Sb.Config.imageValidExtensions.join('|') + ')$', 'i'),
			name		: 'image',
			msgTarget	: 'under',
			buttonText	: SbGetLocaleByKey('imageEditDialogChange'),
			buttonOnly	: true
		});
		uploadFrm.add(this.uploadFld);

		leftColumn.add([
			this.previewLbl, previewCtn, sizeInPxLbl,
			this.widthFld, xLbl, this.heightFld,
			uploadFrm
		]);

		// Right column controls
		/**
		 * Change image button
		 * @type Ext.ux.form.FileUploadField
		 * @property uploadFld
		 */
		this.alignCtl	= new Ext.form.RadioGroup({
			cls				: 'sb-dialog-image-position',
			fieldLabel		: SbGetLocaleByKey('imageEditDialogTextWrapping'),
			labelSeparator	: '',
			hideLabels		: true,
			value			: 'left',
			items			: []
		});
		/**
		 * Possible align value
		 *
		 * @type Array
		 * @property linkStates
		 */
		this.alignValues	= ['left', 'right', 'center'];
		Ext.each(this.alignValues, function (alignValue, index) {
			this.alignCtl.items.push(new Ext.form.Radio({
				inputValue		: alignValue,
				name			: 'alignValue',
				hideLabel		: true,
				width			: 71,
				height			: 61,
				itemCls			: 'sb-dialog-image-position-' + alignValue
			}));
		}, this);

		this.altTextFld		= new Ext.form.TextArea({
			fieldLabel		: SbGetLocaleByKey('imageEditDialogAlternativeTextLabel'),
			itemCls			: 'sb-dialog-image-alt-text',
			anchor			: '100%',
			height			: 44
		});
		this.defaultButton	= this.altTextFld;
		var altTextLbl		= new Ext.Container({
			cls				: 'sb-editor-box-description',
			html			: SbGetLocaleByKey('imageEditDialogAlternativeTextDescription')
		});

		rightColumn.add([
			this.alignCtl, this.altTextFld, altTextLbl
		]);

		Sb.editor.ImageEditDialog.superclass.initComponent.call(this);
	},

	/**
	 * Init events
	 */
	initEvents: function() {
		Sb.editor.ImageEditDialog.superclass.initEvents.call(this);

		this.model.on('elementchange', this.fillDialogFromModel, this);
		if (null != this.domEl) {
			this.model.setElement(this.domEl);
		}

		this.widthFld.on('valid'			, this.onWidthChange, this);
		this.heightFld.on('valid'			, this.onHeightChange, this);
		this.model.on('heightchange'		, this.afterHeightChange, this);
		this.uploadFld.on('fileselected'	, this.onFileSelect, this);
		this.alignCtl.on('change'			, this.onPropertyChange, this);
		this.model.on('alignchange'			, this.afterAlignChange, this);
		this.altTextFld.on('valid'			, this.onPropertyChange, this);
	},

	/**
	 * Clean up events
	 */
	onDestroy: function() {
		this.model.un('elementchange'		, this.fillDialogFromModel, this);

		this.widthFld.un('valid'			, this.onWidthChange, this);
		this.heightFld.un('valid'			, this.onHeightChange, this);
		this.model.un('heightchange'		, this.afterHeightChange, this);
		this.uploadFld.un('fileselected'	, this.onFileSelect, this);
		this.alignCtl.un('change'			, this.onPropertyChange, this);
		this.model.un('alignchange'			, this.afterAlignChange, this);
		this.altTextFld.un('valid'			, this.onPropertyChange, this);

		this.model.selectInEditor();
		this.model.destroy();
	},

	/**
	 * Init from model to dialog
	 */
	fillDialogFromModel: function() {
		if (null === this.memento) {
			this.memento = this.model.saveToMemento();
			if (this.isUploaded) {
				this.model.setProperty('alignTo', 'left');
			}
		}

		var d = this.model.data;
		this.previewLbl.setText(SbGetLocaleByKey('imageEditDialogPreview') + ': ' + d.sourceFileName);
		this.previewLbl.el.ellipsize();
		this.previewImg.el.set({src: d.sourceUrl});
		this.widthFld.setValue(d.width);
		this.heightFld.setValue(d.height);
		this.alignCtl.setValue(d.alignTo);
		this.altTextFld.setValue(d.alternativeText);
	},

	/**
	 * Updates width & refreshes height field
	 */
	onWidthChange: function() {
		this.model.setProperty('width', this.widthFld.getValue());
		this.heightFld.setRawValue(this.model.data.height);
	},

	/**
	 * Updates height & refreshes width field
	 */
	onHeightChange: function() {
		this.model.setProperty('height', this.heightFld.getValue());
		this.widthFld.setRawValue(this.model.data.width);
	},

	/**
	 * Updates simple properties
	 */
	onPropertyChange: function() {
		this.model.unserialize({
			alignTo				: (this.alignCtl.getValue()) ? this.alignCtl.getValue().inputValue : 'none',
			alternativeText		: this.altTextFld.getValue()
		});
	},

	/**
	 * Sets focus after align has changed
	 */
	afterAlignChange: function() {
		this.focus();
	},

	/**
	 * OK button handler
	 */
	okHandler: function() {
		this.destroy();
	},

	/**
	 * Cancel button handler
	 */
	cancelHandler: function() {
		if (this.isUploaded) {
			this.model.removeFromEditor();
			this.afterHeightChange();
		} else {
			this.model.restoreFromMemento(this.memento);
		}
		this.destroy();
		tinyMCE.activeEditor.nodeChanged();
	},

	/**
	 * Fires when user selects image in file upload control
	 */
	onFileSelect: function(field, fieldValue) {
		var frm	= field.ownerCt.getForm();
		if (!frm.isValid()) {
			field.markInvalid(SbGetLocaleByKey('imageEditDialogChangeWarningInvalidType'));
			return;
		}
		frm.submit({
			url			: 'Ajax/Editor/ImageUpload',
			waitMsg		: SbGetLocaleByKey('uploadingImage'),
			success		: function(fp, action) {
				var r		= action.result;
				var root	= Sb.Editor.model.getUrl();
				this.model.unserialize({
					sourceUrl			: root + '/' + Sb.Config.imagePath + '/' + encodeURIComponent(r.sourceFileName),
					heightProportion	: null,
					width				: r.width,
					height				: r.height
				});
				this.fillDialogFromModel();
				if (this.hidden) {
					this.show();
				}
				frm.el.dom.reset();
			}.createDelegate(this),
			failure		: function(fp, action) {
				field.markInvalid(SbGetLocaleByKey('imageEditDialogUploadError'));
			}
		});
	}
});