/**
 * @class Sb.editor.PagesPanel
 * @extends Sb.editor.BaseFormPanel
 * Site pages panel
 * @constructor
 * @param {Object} config The config object
 * @xtype pagespanel
 */
Sb.editor.PagesPanel = Ext.extend(Sb.editor.BaseFormPanel, {

	// private
	initComponent: function() {
		this.treeSelectionModel = new Ext.tree.DefaultSelectionModel({
			listeners: {
				scope: this,
				selectionchange: function(sm, node) {
					if (!node) {
						return;
					}
					this.getForm().setValues(node.attributes.model.data);
				}
			}
		});

		Ext.apply(this, {
			cls: 'sb-editor-pages-panel',
			title: SbGetLocaleByKey('pagesTitle'),
			helperMessage: SbGetLocaleByKey('pagesDescription'),
			tabTip: SbGetLocaleByKey('pagesDescriptionOnHover'),
			labelAlign: 'top',
			layout: 'column',
			autoHeight: true,
			defaults: {
				border: false,
				buttonAlign: 'left'
			},
			items: [{
				columnWidth: 0.6,
				xtype: 'treepanel',
				id: 'sb-pages-tree',
				baseCls: 'sb-pages-tree',
				autoScroll: true,
				animate: true,
				containerScroll: true,
				enableDD: true,
				lines: false,
				loader: new Ext.tree.TreeLoader({
					doPreload: function(node) {
						node.beginUpdate();
						node.attributes.model.pages.each(function(page) {
							var attr = {
								expanded: true,
								id: page.data.identity,
								text: page.data.title,
								iconCls: 'sb-text-icon',
								cls: page.data.hidden ? 'sb-pages-tree-node-hidden' : '',
								model: page,
								maxDepth: 2,
								uiProvider: Sb.editor.PagesTreeNodeUI
							};
							var newNode = page.pages.getCount() == 0
								? new Ext.tree.TreeNode(attr)
								: new Ext.tree.AsyncTreeNode(attr);
							node.appendChild(newNode);
						});
						node.endUpdate();
						return true;
					}
				}),
				rootVisible: false,
				root: {
					id: Sb.Editor.model.rootPage.data.identity,
					model: Sb.Editor.model.rootPage
				},
				selModel: this.treeSelectionModel,
				listeners: {
					movenode: function(tree, node, oldParent, newParent, index) {
						newParent.attributes.model.insert(
							index,
							node.attributes.model,
							oldParent.attributes.model
						);
						Sb.editor.UiContext.registerChanges();
						node.select();
					},
					startdrag: function(panel, node, e) {
						var el = panel.dragZone.proxy.el;
						el.disableShadow();
						el.setXY(el.getXY());
						var ghost = panel.dragZone.proxy.ghost;
						ghost.addClass('sb-pages-tree-drag-ghost');
						ghost.setWidth(Ext.get(node.ui.wrap).getWidth());
					},
					beforemovenode: function(tree, node, oldParent, newParent, index) {
						var lastChildDepth = newParent.getDepth();
						lastChildDepth += 1;
						var nodeDepth = node.getDepth();
						var childrenMaxDepth = 0;
						var computeDepth = function(nodeWithChildren) {
							nodeWithChildren.eachChild(function(childNode) {
								var difference = childNode.getDepth() - nodeDepth;
								if (difference > childrenMaxDepth) {
									childrenMaxDepth = difference;
								}
								computeDepth(childNode);
							});
						};
						computeDepth(node);
						lastChildDepth += childrenMaxDepth;
						return lastChildDepth <= node.attributes.maxDepth;
					}
				},
				buttons: [{
					text: SbGetLocaleByKey('addPage'),
					tooltip: SbGetLocaleByKey('addPageTooltip'),
					iconCls: 'sb-add-icon',
					handler : this.createPage.createDelegate(this, ['text'])
				}, {
					text: SbGetLocaleByKey('deletePage'),
					tooltip: SbGetLocaleByKey('deletePageTooltip'),
					iconCls: 'sb-delete-icon',
					handler : this.deletePage.createDelegate(this)
				}]
			}, {
				columnWidth: 0.4,
				layout: 'form',
				baseCls: 'sb-pages-page-properties',
				layoutConfig: {
					labelSeparator: ''
				},
				defaults: {
					validateOnBlur: false
				},
				items: [{
					xtype: 'textfield',
					fieldLabel: SbGetLocaleByKey('pageTitle'),
					name: 'title',
					allowBlank: false,
					maxLength: 40,
					anchor: '100%',
					msgTarget: 'under',
					listeners: {
						valid: this.onChange.createDelegate(this),
						focus: function(){
							Sb.Editor.Ui.mainWindow.setHelperMessage(SbGetLocaleByKey('messagePageTitle'));
						}
					}
				},{
					xtype: 'textarea',
					fieldLabel: SbGetLocaleByKey('pageDescription'),
					name: 'description',
					msgTarget: 'under',
					maxLength: 255,
					anchor: '100%',
					listeners: {
						valid: this.onChange.createDelegate(this),
						focus: function(){
							Sb.Editor.Ui.mainWindow.setHelperMessage(SbGetLocaleByKey('messageMetaDescription'));
						}
					}
				},{
					xtype: 'textarea',
					fieldLabel: SbGetLocaleByKey('pageKeywords'),
					name: 'keywords',
					msgTarget: 'under',
					maxLength: 255,
					anchor: '100%',
					listeners: {
						valid: this.onChange.createDelegate(this),
						focus: function(){
							Sb.Editor.Ui.mainWindow.setHelperMessage(SbGetLocaleByKey('messageMetaKeywords'));
						}
					}
				},{
					xtype: 'checkbox',
					id: 'hidden-page-id',
					name: 'hidden',
					hideLabel: true,
					boxLabel: SbGetLocaleByKey('pageHidden'),
					listeners: {
						check: this.onChange.createDelegate(this),
						afterrender: function() {
							var fn = Sb.Editor.Ui.mainWindow.setHelperMessage.createDelegate(
								Sb.Editor.Ui.mainWindow, [SbGetLocaleByKey('hiddenPageTooltip')]
							);
							this.el.on('mouseenter', fn);
							this.el.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);

							this.labelEl.on('mouseenter', fn);
							this.labelEl.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);
						}
					}
				}],
				buttons: [{
					text: SbGetLocaleByKey('openPage'),
					tooltip: SbGetLocaleByKey('openPageTooltip'),
					iconCls: 'sb-open-page-icon',
					scope: this,
					handler: function() {
						Sb.Editor.siteView.setCurrentPage(
							this.treeSelectionModel.getSelectedNode().attributes.model
						);
					}
				}]
			}]
		});

		Sb.editor.PagesPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Activate handler
	 */
	onActivate: function() {
		this.memento = Sb.Editor.model.rootPage.saveToMemento();
		var rootNode = Ext.getCmp('sb-pages-tree').getRootNode();
		rootNode.reload();
		// select current page in tree
		rootNode.findChild('id', Sb.Editor.siteView.currentPage.data.identity, true).select();

		Sb.editor.PagesPanel.superclass.onActivate.apply(this, arguments);
	},

	/**
	 * Change handler
	 *
	 * @param {Ext.form.Field} field
	 */
	onChange: function(field) {
		var node = this.treeSelectionModel.getSelectedNode();
		switch (field.name) {
			case 'title':
				node.setText(Ext.util.Format.htmlEncode(
					Ext.util.Format.ellipsis(field.getValue(), 35 - node.getDepth() * 2)
				));
				break;
			case 'hidden':
				node.ui[(field.getValue() ? 'add' : 'remove') + 'Class']('sb-pages-tree-node-hidden');
				break;
		}

		if (node.attributes.model.setProperty(field.name, field.getValue())) {
			Sb.editor.UiContext.registerChanges();
		}
	},

	/**
	 * Create page by type
	 * @param {String} type
	 */
	createPage: function(type) {
		var currentNode = this.treeSelectionModel.getSelectedNode();
		var	newPage = currentNode.attributes.model.createPage();
		var	newPageNode;
		switch (type) {
			case 'text':
				newPage.data.title = SbGetLocaleByKey('newTextPage');

				this.addTextWidget(newPage, 'content');
				this.addTextWidget(newPage, 'column1');
				this.addTextWidget(newPage, 'column2');

				newPageNode = new Ext.tree.TreeNode({
					id: newPage.data.identity,
					text: newPage.data.title,
					iconCls: 'sb-text-icon',
					model: newPage,
					maxDepth: 2,
					uiProvider: Sb.editor.PagesTreeNodeUI
				});
				break;
		}
		currentNode.parentNode.insertBefore(newPageNode, currentNode.nextSibling);
		currentNode.parentNode.attributes.model.insertChild(newPage, currentNode.parentNode.indexOf(newPageNode));
		Sb.editor.UiContext.registerChanges();
		newPageNode.select();
	},

	/**
	 * Add new text widget with content "Type content here"
	 * @param {Sb.model.SitePage} page
	 * @param {String} region
	 */
	addTextWidget: function(page, region) {
		var widgetContainer = page.widget.widgets.find(function(container) {
			return container.data.region == region;
		});

		var contentWidget = new Sb.model.Widget({
			type: 'text',
			content: SbGetLocaleByKey('typeContentHere')
		});

		widgetContainer.widgets.insert(0, contentWidget);
	},

	/**
	 * Delete page from model and tree
	 * @return {Boolean}
	 */
	deletePage: function() {
		var rootNode = Ext.getCmp('sb-pages-tree').getRootNode();
		var nodeForDelete = this.treeSelectionModel.getSelectedNode();
		if (!nodeForDelete) {
			return false;
		}
		if ((nodeForDelete.parentNode === rootNode) && (rootNode.childNodes.length == 1)) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageInfo(SbGetLocaleByKey('cannotRemoveAllPages'));
			return false;
		}

		var nodeForSelect = nodeForDelete.nextSibling || nodeForDelete.previousSibling || nodeForDelete.parentNode;

		nodeForDelete.parentNode.attributes.model.pages.remove(nodeForDelete.attributes.model);
		nodeForDelete.parentNode.removeChild(nodeForDelete);

		nodeForDelete.attributes.model.applySelfAndRecursive(function (page) {
			if (page.data.identity == Sb.Editor.siteView.currentPage.data.identity) {
				Sb.Editor.siteView.setCurrentPage(
					nodeForSelect.attributes.model
				);
			}
		});
		nodeForSelect.select();

		Sb.editor.UiContext.registerChanges();
		return true;
	},

	/**
	 * OK button handler.
	 */
	okHandler: function() {
		if (!this.form.isValid()) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
				SbGetLocaleByKey('fillRequiredFields')
			);
			return;
		}
		Sb.editor.PagesPanel.superclass.okHandler.call(this);
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function(afterSaveCallbacks) {
		Sb.Editor.model.rootPage.save({
			success: function() {
				afterSaveCallbacks.success.call();
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					SbGetLocaleByKey('messageSitemapChangesSaved')
				);
			},
			failure: afterSaveCallbacks.failure
		});
	},

	/**
	 * Discard handler
	 */
	discardHandler: function() {
		Sb.Editor.model.rootPage.restoreFromMemento(this.memento);
	}
});
Ext.reg('pagespanel', Sb.editor.PagesPanel);