/**
 * @class Sb.editor.StylePanel
 * @extends Sb.editor.BaseFormPanel
 * Style editor panel
 * @constructor
 * @param {Object} config The config object
 * @xtype stylepanel
 */
Sb.editor.StylePanel = Ext.extend(Sb.editor.BaseFormPanel, {

	// private
	initComponent: function() {
		var designSettings = Sb.Editor.model.design.serialize();

		var schemes = [];
		Ext.iterate(Sb.Editor.model.data.template.colorschemes, function(key, value) {
			schemes.push({
				id: key,
				title: SbGetLocaleByKey('Styleset' + key),
				colors: value
			});
		});
		var schemesStore = new Ext.data.JsonStore({
			fields: ['id', 'title', 'colors'],
			data: schemes
		});

		var schemeTab = {
			itemId: 'scheme',
			title: SbGetLocaleByKey('styleSchemeTab'),
			helperMessage: SbGetLocaleByKey('styleSchemeTabDescription'),
			cls: 'sb-editor-style-scheme-panel',
			layout: 'form',
			defaults: {border: false},
			items: [{
				layout: 'form',
				items: [{
					xtype: 'combo',
					id: 'colorschemes',
					hideLabel: true,
					triggerAction: 'all',
					mode: 'local',
					store: schemesStore,
					anchor: '100%',
					editable: false,
					displayField: 'title',
					valueField: 'id',
					tpl: new Ext.XTemplate(
						'<tpl for="."><div class="x-combo-list-item sb-styleset-list-item">' +
							'<div style="float: left;">{title}</div>' +
							'<ul class="preview">' +
								'<li class="first" style="background-color: {[values.colors.color1]};"></li>' +
								'<li style="background-color: {[values.colors.color2]};"></li>' +
								'<li style="background-color: {[values.colors.color3]};"></li>' +
								'<li class="last" style="background-color: {[values.colors.color4]};"></li>' +
							'</ul>' +
						'</div></tpl>'
					),
					listeners: {
						scope: this,
						select: function(combo) {
							var record = combo.findRecord(combo.valueField, combo.getValue());
							var scheme = record.get('colors');

							this.form.findField('baseColor1').setRawValue(scheme.color1);
							this.form.findField('baseColor2').setRawValue(scheme.color2);
							this.form.findField('baseColor3').setRawValue(scheme.color3);
							this.form.findField('baseColor4').setRawValue(scheme.color4);
							this.selectedColorField.updateCurrentPicker();

							this.onColorSchemeChange();
						}
					}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-style-description',
					html		: SbGetLocaleByKey('styleSchemeTabLongDescription')
				}]
			}, {
				layout: 'column',
				items:[{ // Column 1
					width: 195,
					cls: 'sb-editor-panel-column-margin',
					id: 'schemeColorPicker',
					xtype: 'colorpicker',
					listeners: {
						scope: this,
						change: function(colorpicker, value) {
							this.selectedColorField.setRawValue(value, true);
							this.lastChangeTime = this.lastChangeTime || (new Date).getTime();
							if (Ext.isIE7) {
								return;
							}
							var delay = 0, taskDelay = 0;
							if (!Ext.isIE) {
								delay = 100;
								taskDelay = 20;
							} else {
								delay = 250;
							}
							if (delay > 0) {
								var now = (new Date).getTime();
								var difference = now - this.lastChangeTime;
								if (difference <= delay) {
									return;
								}
								this.lastChangeTime = now;
							}
							if (taskDelay) {
								var task = new Ext.util.DelayedTask(this.onColorFieldChange, this, [this.selectedColorField]);
								task.delay(taskDelay);
							} else {
								this.onColorFieldChange(this.selectedColorField);
							}
						},
						mouseup: function(colorpicker, value) {
							if (!Ext.isIE7) {
								return;
							}
							this.onColorFieldChange(this.selectedColorField);
						}
					}
				}, { // Column 2
					columnWidth: 1,
					layout: 'form',
					border: false,
					labelAlign: 'right',
					style: {paddingTop: '25px'},
					defaults: {
						xtype			: 'colorfield',
						currentPicker	: 'schemeColorPicker',
						listeners: {
							scope		: this,
							focus		: this.selectColorField,
							valid		: this.onColorFieldChange,
							invalid		: Sb.editor.UiContext.registerChanges
						}
					},
					items: [{
						name: 'baseColor1',
						fieldLabel: SbGetLocaleByKey('color') + ' 1'
					}, {
						name: 'baseColor2',
						fieldLabel: SbGetLocaleByKey('color') + ' 2'
					}, {
						name: 'baseColor3',
						fieldLabel: SbGetLocaleByKey('color') + ' 3'
					}, {
						name: 'baseColor4',
						fieldLabel: SbGetLocaleByKey('color') + ' 4'
					}]
				}]
			}],
			listeners: {
				scope		: this,
				activate	: {
					fn		: this.loadSchemeTab,
					single	: true
				}
			}
		};

		var fillsTpl = new Ext.XTemplate(
			'<tpl for=".">',
				'<div id="{name}" class="sb-selectbox-list-item">',
					'<img style="background: {[this.backgroundColor]}" src="',
						'<tpl if="name == \'background-none\'">{[Ext.BLANK_IMAGE_URL]}</tpl>',
						'<tpl if="name != \'background-none\'">' + Sb.Config.templatesUrl + '/' +
							Sb.Editor.model.data.template.code + '/decorations/background/{name}/images/{name}.png</tpl>',
					'" alt="{name}">',
				'</div>',
			'</tpl>'
		);

		this.fillStore = new Ext.data.ArrayStore({
			fields: ['name']
		});

		var colorsTab = {
			itemId: 'colors',
			title: SbGetLocaleByKey('styleColorsTab'),
			helperMessage: SbGetLocaleByKey('styleColorsTabDescription'),
			cls: 'sb-editor-style-colors-panel',
			layout: 'column',
			defaults: {border: false},
			items: [{ // Column 1
				width: 195,
				cls: 'sb-editor-panel-column-margin',
				defaults: {
					layout: 'form',
					border: false
				},
				items: [{
					labelAlign: 'left',
					items: [{
						xtype: 'combo',
						itemCls: 'sb-style-fill-target',
						name: 'bgPlace',
						mode: 'local',
						anchor: '100%',
						hideLabel: true,
						editable: false,
						triggerAction: 'all',
						lazyInit: false,
						store: new Ext.data.ArrayStore({
							fields: ['value', 'name']
						}),
						displayField: 'name',
						valueField: 'value',
						autoSelect: true,
						listeners: {
							scope: this,
							beforeselect: this.onBgPlaceBeforeSelect,
							select: function(combo) {
								this.onBgPlaceSelect(combo.getValue());
							}
						}
					}, {
						xtype: 'colorpicker',
						id: 'colorsTabColorPicker',
						listeners: {
							scope: this,
							change: function(colorpicker, value) {
								this.selectedColorsTabColorField.setRawValue(value, true);
								colorpicker.lastChangeTime = colorpicker.lastChangeTime || (new Date).getTime();
								if (Ext.isIE7) {
									this.onUpdatePreview();
									return;
								}
								var delay = 0, taskDelay = 0;
								if (!Ext.isIE) {
									delay = this.isContainerContent() ? 0 : 100;
									taskDelay = this.isContainerContent() ? 0 : 20;
								}
								if (delay > 0) {
									var now = (new Date).getTime();
									var difference = now - colorpicker.lastChangeTime;
									if (difference <= delay) {
										return;
									}
									this.lastChangeTime = now;
								}
								if (taskDelay) {
									var task = new Ext.util.DelayedTask(this.onColorsTabColorFieldChange, this, [this.selectedColorsTabColorField]);
									task.delay(taskDelay);
								} else {
									this.onColorsTabColorFieldChange(this.selectedColorsTabColorField);
								}
							},
							mouseup: function(colorpicker, value) {
								if (!Ext.isIE7) {
									return;
								}
								this.onColorsTabColorFieldChange(this.selectedColorsTabColorField);
							}
						}
					}]
				}, {
					labelAlign: 'top',
					items: [{
						xtype: 'compositefield',
						fieldLabel: SbGetLocaleByKey('backgroundColor'),
						combineErrors: false,
						items: [this.generateColorsTabColorField({
							name: 'backgroundColor'
						}), this.generateColorsTabColorAutoField({
							name: 'backgroundColorAuto'
						})]
					}, {
						xtype: 'sbselectbox',
						fieldLabel: SbGetLocaleByKey('backgroundGradient'),
						name: 'backgroundGradient',
						anchor: '100%',
						mode: 'local',
						store: this.fillStore,
						lazyInit: false,
						valueField: 'name',
						cls: 'sb-gradient-images',
						listClass: 'sb-gradient-images-list',
						itemSelector: 'div.sb-selectbox-list-item',
						listWidth: 343,
						tpl: fillsTpl,
						setBackgroundColor: function(color) {
							this.tpl.backgroundColor = color;
							this.el.select('.sb-selectbox-list-item img').setStyle('background', color);
							Ext.iterate(this.view.getNodes(), function(item) {
								item.firstChild.style.background = color;
							});
						},
						doFilter: function() {
							var numbers = '';
							var color = Ext.getCmp('invert-color-black').checked ? 'black' : 'white';

							if (Ext.getCmp('filter-vertical').checked) {
								numbers += '|2|3|7|10|11|14|15|18|19|22|23';
							}
							if (Ext.getCmp('filter-horizontal').checked) {
								numbers += '|4|6|8|9|12|13|16|17|20|21|24|25';
							}
							this.store.filterBy(function(record) {
								var name = record.get('name');
								if (name == 'background-none') {
									return true;
								}
								var filterRe = new RegExp('background(' + numbers + ')-' + color, 'i');
								return filterRe.test(name);
							});
							this.restrictHeight();
							if (!this.selectByValue(this.getValue(), true)) {
								this.select(this.selectedIndex, true);
								var records = this.view.getSelectedRecords();
								if (records && records.length) {
									this.setValue(records[0].data[this.valueField]);
									this.fireEvent('select', this, records[0], this.selectedIndex);
								}
							}
						},
						lastQuery: '',
						listeners: {
							scope: this,
							select: function(selectbox) {
								var bgPlace = this.form.findField('bgPlace').getValue();
								Sb.Editor.model.design.setDecoration(
									'background', bgPlace + 'Background', selectbox.getValue()
								);
								Sb.editor.UiContext.registerChanges();
							},
							afterrender: function(selectbox) {
								selectbox.header = selectbox.list.createChild(
									{cls: 'x-combo-list-hd'},
									selectbox.innerList
								);
								new Ext.Container({
									renderTo: selectbox.header,
									defaultType: 'container',
									items: [{
										cls: 'sb-invert-colors',
										defaultType: 'radio',
										items: [{
											xtype: 'label',
											text: SbGetLocaleByKey('invertBorderColors') + ':',
											cls: 'sb-invert-colors-title'
										}, {
											id: 'invert-color-black',
											name: 'invertBackgroundColors',
											boxLabel: SbGetLocaleByKey('black'),
											checked: true,
											listeners: {
												scope: selectbox,
												check: selectbox.doFilter
											}
										}, {
											id: 'invert-color-white',
											name: 'invertBackgroundColors',
											boxLabel: SbGetLocaleByKey('white')
										}]
									}, {
										cls: 'sb-filter-by-direction',
										defaultType: 'checkbox',
										items: [{
											xtype: 'label',
											text: SbGetLocaleByKey('filterByDirection') + ':',
											cls: 'sb-filter-by-direction-title'
										}, {
											id: 'filter-vertical',
											boxLabel: SbGetLocaleByKey('vertical'),
											checked: true,
											listeners: {
												scope: selectbox,
												check: selectbox.doFilter
											}
										}, {
											id: 'filter-horizontal',
											boxLabel: SbGetLocaleByKey('horizontal'),
											checked: true,
											listeners: {
												scope: selectbox,
												check: selectbox.doFilter
											}
										}]
									}]
								});
								selectbox.assetHeight += selectbox.header.getHeight();
							}
						}
					}]
				}]
			}, {
				columnWidth: 1,
				layout: 'form',
				id: 'sb-style-color-additional-column',
				labelAlign: 'right',
				items: [{
					xtype: 'label',
					cls: 'sb-editor-panel-heading',
					text: SbGetLocaleByKey('fontColors')
				}, {
					xtype: 'compositefield',
					fieldLabel: SbGetLocaleByKey('textColorLabel'),
					combineErrors: false,
					itemCls: 'sb-colorfield-area',
					items: [this.generateColorsTabColorField({
						name: 'textColor'
					}), this.generateColorsTabColorAutoField({
						name: 'textColorAuto'
					})]
				}, {
					xtype: 'compositefield',
					fieldLabel: SbGetLocaleByKey('linkColorLabel'),
					combineErrors: false,
					itemCls: 'sb-colorfield-area',
					items: [this.generateColorsTabColorField({
						name: 'linkColor'
					}), this.generateColorsTabColorAutoField({
						name: 'linkColorAuto'
					})]
				}, {
					xtype: 'compositefield',
					fieldLabel: SbGetLocaleByKey('h1ColorLabel'),
					combineErrors: false,
					itemCls: 'sb-colorfield-area',
					items: [this.generateColorsTabColorField({
						name: 'h1Color'
					}), this.generateColorsTabColorAutoField({
						name: 'h1ColorAuto'
					})]
				}, {
					xtype: 'compositefield',
					fieldLabel: SbGetLocaleByKey('h2ColorLabel'),
					combineErrors: false,
					itemCls: 'sb-colorfield-area',
					items: [this.generateColorsTabColorField({
						name: 'h2Color'
					}), this.generateColorsTabColorAutoField({
						name: 'h2ColorAuto'
					})]
				}, {
					border: false,
					layout: 'form',
					labelAlign: 'top',
					items: [{
						xtype: 'displayfield',
						name: 'colorsPreview',
						id: 'preview-color-background',
						cls: 'x-form-text',
						style: {
							'overflow': 'hidden'
						},
						anchor: '0',
						height: 166,
						value:	'<div class="content">' +
									'<div class="sb-widget-text-color">' +
										'<h1 id="preview-color-h1">' + SbGetLocaleByKey('previewHeading') + ' 1</h1>' +
										'<h2 id="preview-color-h2">' + SbGetLocaleByKey('previewHeading') + ' 2</h2>' +
										'<p id="preview-color-text">' + SbGetLocaleByKey('previewText') + '</p>' +
									'</div>' +
								'</div>',
						fieldLabel: SbGetLocaleByKey('preview')
					}]
				}]
			}],
			listeners: {
				scope		: this,
				activate	: {
					fn		: this.loadColorsTab,
					single	: true
				}
			}
		};

		var fontStorage = new Ext.data.ArrayStore({
			fields: ['value'],
			data: [['Andale Mono'], ['Arial'], ['Arial Black'], ['Book Antiqua'], ['Comic Sans MS'], ['Courier New'],
				['Georgia'], ['Helvetica'], ['Impact'], ['Symbol'], ['Tahoma'], ['Terminal'], ['Times New Roman'],
				['Trebuchet MS'], ['Verdana'], ['Webdings'], ['Wingdings']]
		});

		var fontSizeStorage = new Ext.data.ArrayStore({
			fields: ['value'],
			data: [['8'], ['9'], ['10'], ['11'], ['12'], ['14'], ['16'], ['18'], ['20'], ['22'], ['24'], ['26'], ['28'], ['36'], ['48'], ['72']]
		});

		var fontSizeComboCfg = {
			fieldLabel: SbGetLocaleByKey('fontSize'),
			xtype: 'combo',
			editable: false,
			anchor: '90%',
			triggerAction: 'all',
			mode: 'local',
			store: fontSizeStorage,
			valueField: 'value',
			displayField: 'value',
			editable: true,
			maskRe: /[0-9]/,
			listeners: {
				scope: this,
				select: this.onFontFieldChange,
				change: this.onFontFieldChange,
				specialkey: function(combo, e) {
					if (e.getKey() == e.ENTER) {
						this.onFontFieldChange(combo);
					}
				}
			}
		};

		var fontComboCfg = {
			fieldLabel: SbGetLocaleByKey('fontFamily'),
			xtype: 'combo',
			editable: false,
			anchor: '95%',
			triggerAction: 'all',
			mode: 'local',
			store: fontStorage,
			valueField: 'value',
			displayField: 'value',
			tpl: new Ext.XTemplate(
				'<tpl for="."><div class="x-combo-list-item"' +
					'<tpl if="value!=\'Webdings\' && value!=\'Wingdings\'"> style="font-family:{value}"</tpl>' +
				'>{value}</div></tpl>'
			),
			listeners: {
				select: this.onFontFieldChange.createDelegate(this)
			}
		};

		var fontsTab = {
			itemId: 'fonts',
			title: SbGetLocaleByKey('styleFontsTab'),
			cls: 'sb-editor-style-fonts-panel',
			layout: 'column',
			defaults: {border: false, layout: 'form'},
			helperMessage: SbGetLocaleByKey('styleFontsTabDescription'),
			items: [{ // Column 1
				columnWidth: 0.6,
				cls: 'sb-editor-panel-column-margin',
				defaults: {border: false},
				items: [{
					xtype: 'label',
					text: SbGetLocaleByKey('h1ColorLabel'),
					cls: 'sb-editor-panel-heading'
				}, {
					layout: 'column',
					defaults: {
						border: false,
						layout: 'form',
						labelAlign: 'top'
					},
					items: [{
						columnWidth: 0.5,
						items: Ext.apply({name: 'h1Font'}, fontComboCfg)
					}, {
						columnWidth: 0.2,
						items: Ext.apply({name: 'h1FontSize'}, fontSizeComboCfg)
					}, {
						columnWidth: 0.3,
						items: this.generateFontTabDecorationsField('h1')
					}]
				}, {
					xtype: 'label',
					text: SbGetLocaleByKey('h2ColorLabel'),
					cls: 'sb-editor-panel-heading'
				}, {
					layout: 'column',
					defaults: {
						border: false,
						layout: 'form',
						labelAlign: 'top'
					},
					items: [{
						columnWidth: 0.5,
						items: Ext.apply({name: 'h2Font'}, fontComboCfg)
					}, {
						columnWidth: 0.2,
						items: Ext.apply({name: 'h2FontSize'}, fontSizeComboCfg)
					}, {
						columnWidth: 0.3,
						items: this.generateFontTabDecorationsField('h2')
					}]
				}, {
					xtype: 'label',
					text: SbGetLocaleByKey('textColorLabel'),
					cls: 'sb-editor-panel-heading'
				}, {
					layout: 'column',
					defaults: {
						border: false,
						layout: 'form',
						labelAlign: 'top'
					},
					items: [{
						columnWidth: 0.5,
						items: Ext.apply({name: 'textFont'}, fontComboCfg)
					}, {
						columnWidth: 0.2,
						items: Ext.apply({name: 'textFontSize'}, fontSizeComboCfg)
					}]
				}]
			}, { // Column 2
				columnWidth: 0.4,
				labelAlign: 'top',
				items: [{
					xtype: 'displayfield',
					name: 'fontPreview',
					cls: 'x-form-text',
					style: {
						'overflow': 'hidden'
					},
					anchor: '0',
					height: 174,
					value:	'<div class="pageContentText">' +
								'<h1>' + SbGetLocaleByKey('previewHeading') + ' 1</h1>' +
								'<h2>' + SbGetLocaleByKey('previewHeading') + ' 2</h2>' +
								'<p>' + SbGetLocaleByKey('previewText') + '</p>' +
							'</div>',
					fieldLabel: SbGetLocaleByKey('preview')
				}]
			}],
			listeners: {
				scope		: this,
				activate	: {
					fn		: this.loadFontsTab,
					single	: true
				}
			}
		};

		var styleBorderTab = {
			itemId: 'styleBorder',
			xtype: 'styleborderpanel'
		};

		Ext.apply(this, {
			cls: 'sb-editor-style-panel',
			title: SbGetLocaleByKey('styleTitle'),
			helperMessage: SbGetLocaleByKey('styleDescription'),
			tabTip: SbGetLocaleByKey('styleDescriptionOnHover'),
			autoHeight: true,
			items: {
				xtype: 'verticaltabpanel',
				tabWidth: 85,
				tabPosition: 'left',
				activeTab: 0,
				border: false,
				cls: 'sb-editor-style-panel-tabs',
				items: [schemeTab, colorsTab, fontsTab, styleBorderTab],
				defaults: {height: Ext.isIE ? 357 : 353},
				listeners: {
					scope: this,
					tabchange: this.onTabChange
				}
			}
		});

		Sb.editor.StylePanel.superclass.initComponent.apply(this, arguments);

		/**
		 * Inner tab panel
		 * @type Ext.TabPanel
		 * @property tabs
		 */
		this.tabs = this.items.item(0);

		this.colorSchemesCombo = Ext.getCmp('colorschemes');
	},

	selectColorField: function(field) {
		this.selectedColorField = field;
	},

	onColorFieldChange: function(field) {
		if (field.selectedStartValue == field.getValue()) {
			return;
		}
		this.colorSchemesCombo.setValue(SbGetLocaleByKey('Stylesetcustom'));
		this.onColorSchemeChange();
	},

	onColorSchemeChange: function() {
		var colors = {colorScheme: {}};
		for (var i = 1; i <= 4; i++) {
			var field = this.form.findField('baseColor' + i);
			if (!field.isValidatedOk) {
				continue;
			}
			colors.colorScheme['color' + i] = field.getValue();
		}
		if (!Sb.Editor.model.design.unserialize(colors)) {
			return;
		}
		Sb.editor.UiContext.registerChanges();
	},

	updatePlacesData: function() {
		var placesData = [];
		var layout = Sb.Editor.model.design.getSettings().layout;
		Ext.iterate(['site', 'watermark', 'header', 'footer', 'content'], function (place) {
			placesData.push([place, SbGetLocaleByKey(place + 'Title')]);
		});
		if (layout.sidebarFirst) {
			placesData.push(['column1',	SbGetLocaleByKey((layout.sidebarFirstPosition == 'left' ? 'column1' : 'column2') + 'Title')]);
		}
		if (layout.sidebarSecond) {
			placesData.push(['column2',	SbGetLocaleByKey((layout.sidebarSecondPosition == 'left' ? 'column1' : 'column2') + 'Title')]);
		}
		this.form.findField('bgPlace').store.loadData(placesData);
	},

	loadSchemeTab: function() {
		var designSettings = Sb.Editor.model.design.getSettings();

		var colorScheme = designSettings.colorScheme;
		this.form.findField('baseColor1').setRawValue(colorScheme.color1, true);
		this.form.findField('baseColor2').setRawValue(colorScheme.color2, true);
		this.form.findField('baseColor3').setRawValue(colorScheme.color3, true);
		this.form.findField('baseColor4').setRawValue(colorScheme.color4, true);

		// search match current color scheme to one in all colors schemes
		this.colorSchemesCombo.setValue(SbGetLocaleByKey('Stylesetcustom'));
		// iterate style sets
		this.colorSchemesCombo.store.each(function(record) {
			// set color set name if match
			if (colorScheme.color1 == record.data.colors.color1
				&& colorScheme.color2 == record.data.colors.color2
				&& colorScheme.color3 == record.data.colors.color3
				&& colorScheme.color4 == record.data.colors.color4
			) {
				this.colorSchemesCombo.setValue(record.data.id);
				return false;
			}
		}, this);

		if (!this.selectedColorField) {
			this.selectedColorField = this.form.findField('baseColor1');
		}
		this.selectedColorField.focus();
	},

	loadFontsTab: function() {
		var designSettings = Sb.Editor.model.design.getSettings();

		this.form.setValues({
			textFont: designSettings.font.textFont,
			textFontSize: designSettings.font.textFontSize,
			h1Font: designSettings.font.h1Font,
			h1FontSize: designSettings.font.h1FontSize,
			h2Font: designSettings.font.h2Font,
			h2FontSize: designSettings.font.h2FontSize
		});
	},

	loadColorsTab: function() {
		Sb.DesignTemplateService.getDecorationVariants('generic', 'background', [], function(result) {
			var data = [];
			Ext.iterate(result, function (item) {
				data.push([item]);
			});
			this.fillStore.loadData(data);
			this.form.findField('backgroundGradient').doFilter();
		}, this);

		this.updatePlacesData();
		Sb.Editor.model.design.on('layoutchange', this.updatePlacesData, this);

		var bgPlaceCombo	= this.form.findField('bgPlace'),
			bgPlace			= bgPlaceCombo.getValue() || 'content';

		Ext.get('sb-style-color-additional-column')[!this.isContainerContent() ? 'hide' : 'show']();
		bgPlaceCombo.setValue(bgPlace);
		this.updateColorsTab();
	},

	updateColorsTab: function() {
		var form					= this.form,
			bgPlace					= form.findField('bgPlace').getValue() || 'content',
			design 					= Sb.Editor.model.design,
			designSettings 			= design.getSettings(),
			fontColorFields			= ['text', 'link', 'h1', 'h2'],
			previewEl				= form.findField('colorsPreview').el,
			backgroundGradientValue	= designSettings.background[bgPlace + 'Background'],
			backgroundColorValue	= designSettings.backgroundColor[bgPlace + 'Color'],
			invertBlack				= Ext.getCmp('invert-color-black'),
			invertWhite				= Ext.getCmp('invert-color-white'),
			reIsWhite				= new RegExp('white'),
			isSwitchToWhite			= invertBlack.checked && reIsWhite.test(backgroundGradientValue),
			isSelectBackground 		= !this.isContainerContent() || !this.selectedColorsTabColorField;

		Ext.each(fontColorFields, function(key) {
			var modelKey = bgPlace + key.toUpperCaseFirst() + 'Color';
			form.findField(key + 'Color').setRawValue(designSettings.fontColor[modelKey], true);
			form.findField(key + 'ColorAuto').setRawValue(!design.customColors[modelKey]);
		});
		form.findField('backgroundColor').setRawValue(backgroundColorValue, true);
		form.findField('backgroundColorAuto').setRawValue(!design.customColors[bgPlace + 'Color']);

		if (previewEl) {
			previewEl.first().dom.className = bgPlace;
			previewEl.setStyle('background-color', backgroundColorValue);
		}
		if (isSwitchToWhite) {
			invertBlack.setValue(false);
			invertWhite.setValue(true);
		}
		form.findField('backgroundGradient').setValue(backgroundGradientValue);
		form.findField('backgroundGradient').setBackgroundColor(backgroundColorValue);

		if (isSelectBackground) {
			this.selectedColorsTabColorField = form.findField('backgroundColor');
		}
		this.selectedColorsTabColorField.focus();

		this.onUpdatePreview();
	},

	onBgPlaceBeforeSelect: function(combo) {
		if (this.showActiveErrors()) {
			combo.collapse();
			return false;
		}
		return true;
	},

	onBgPlaceSelect: function(place) {
		var el			= Ext.get('sb-style-color-additional-column'),
			slideTo		= 'l',
			effect		= {
				easing		: 'easeIn',
				duration	: .3,
				stopFx		: true,
				useDisplay	: false
			};

		if (!this.isContainerContent()) {
			if (el.isVisible()) {
				el.slideOut(slideTo, effect);
			}
		} else {
			if (!el.isVisible()) {
				el.slideIn(slideTo, effect);
			}
		}

		this.updateColorsTab();
	},

	selectColorsTabColorField: function(field) {
		this.selectedColorsTabColorField = field;
	},

	onColorsTabColorFieldChange: function(field) {
		if (field.selectedStartValue == field.getValue()) {
			return;
		}

		var bgPlace = this.form.findField('bgPlace').getValue();
		var section, property;

		this.onUpdatePreview();
		switch (field.getName()) {
			case 'backgroundColor':
				section = 'backgroundColor';
				property = bgPlace + 'Color';
				break;
			case 'textColor':
				section = 'fontColor';
				property = bgPlace + 'TextColor';
				break;
			case 'linkColor':
				section = 'fontColor';
				property = bgPlace + 'LinkColor';
				break;
			case 'h1Color':
				section = 'fontColor';
				property = bgPlace + 'H1Color';
				break;
			case 'h2Color':
				section = 'fontColor';
				property = bgPlace + 'H2Color';
				break;
		}

		var newProperties = {};
		newProperties[property] = field.getValue();
		if (!Sb.Editor.model.design.unserializeObject(section, newProperties)) {
			return;
		}

		if (field.getValue() != Sb.Editor.model.design.palette[property]) {
			Sb.editor.UiContext.registerChanges();
		}
		this.form.findField(field.getName() + 'Auto').setRawValue(false);
	},

	onColorsTabColorAutoFieldCheck: function(field, checked) {
		var bgPlace = this.form.findField('bgPlace').getValue();
		var section, property;
		switch (field.getName()) {
			case 'backgroundColorAuto':
				section = 'backgroundColor';
				property = bgPlace + 'Color';
				break;
			case 'textColorAuto':
				section = 'fontColor';
				property = bgPlace + 'TextColor';
				break;
			case 'linkColorAuto':
				section = 'fontColor';
				property = bgPlace + 'LinkColor';
				break;
			case 'h1ColorAuto':
				section = 'fontColor';
				property = bgPlace + 'H1Color';
				break;
			case 'h2ColorAuto':
				section = 'fontColor';
				property = bgPlace + 'H2Color';
				break;
		}

		var field = this.form.findField(field.getName().replace('Auto', ''));
		if (checked) {
			var newProperties = {};
			newProperties[property] = '';
			Sb.Editor.model.design.unserializeObject(section, newProperties);
			field.setValue(Sb.Editor.model.design.data[section][property]);
			this.onUpdatePreview();
		} else {
			Sb.Editor.model.design.customColors[property] = true;
		}
		Sb.editor.UiContext.registerChanges();
		field.focus();
	},

	/**
	 * Generates common config for color tab color field
	 *
	 * @param {Object} Config
	 */
	generateColorsTabColorField: function(config) {
		return Ext.apply({
			xtype			: 'colorfield',
			currentPicker	: 'colorsTabColorPicker',
			listeners		: {
				scope		: this,
				focus		: this.selectColorsTabColorField,
				valid		: this.onColorsTabColorFieldChange,
				invalid		: Sb.editor.UiContext.registerChanges
			}
		}, config);
	},

	/**
	 * Generates common config for color tab auto checkbox
	 *
	 * @param {Object} Config
	 */
	generateColorsTabColorAutoField: function(config) {
		return Ext.apply({
			xtype: 'checkbox',
			boxLabel: SbGetLocaleByKey('styleColorsAutoLabel'),
			listeners: {
				scope: this,
				check: this.onColorsTabColorAutoFieldCheck,
				afterrender: function(me) {
					Ext.QuickTips.registerSelfAndChildren({
						target: me, text: SbGetLocaleByKey('styleColorsAutoHelper')
					});
				}
			}
		}, config);
	},

	/**
	 * Generates common config for font decorations
	 *
	 * @param {String} prefix
	 */
	generateFontTabDecorationsField: function(prefix) {
		var panel = new Sb.form.ColorDecorationPanel({
			itemId			: prefix + 'Decoration',
			cls				: 'sb-edit-control clearfix',
			widgetContolCls	: 'sb-edit-control-',
			children		: 'decoration',
			decorations		: ['Bold', 'Italic']
		});
		var decorationPanel	= panel.getComponent('Decoration');
		var model			= Sb.Editor.model.design;
		Ext.each(panel.decorations, function(decoration) {
			var button		= decorationPanel.getComponent(decoration);
			var modelKey	= prefix + 'Font' + decoration.toUpperCaseFirst();
			this.on('render', function() {
				this.tabs.getComponent('fonts').on('activate', function() {
					button.toggle(1 == model.data.font[modelKey], true);
				});
			}, this);
			button.on('toggle', function() {
				var newProperties				= {font: {}};
				newProperties.font[modelKey]	= this.pressed;
				model.unserialize(newProperties);
				Sb.editor.UiContext.registerChanges();
			});
		}, this);

		return panel;
	},

	/**
	 * Fires after fonts form field has changed.
	 * Sets new design settings values
	 *
	 * @param {Object} Text field or combobox
	 */
	onFontFieldChange: function(field) {
		var newProperties = {font: {}};
		newProperties.font[field.getName()] = field.getRawValue();

		Sb.Editor.model.design.unserialize(newProperties);
		Sb.editor.UiContext.registerChanges();
	},

	/**
	 * Activate handler
	 */
	onActivate: function() {
		this.memento = Sb.Editor.model.design.saveToMemento();
		this.tabs.activeTab.fireEvent('activate', this.tabs.activeTab);
		Sb.editor.StylePanel.superclass.onActivate.apply(this, arguments);
	},

	/**
	 * Fires after the active tab has changed.
	 * @param {TabPanel} tabPanel
	 * @param {Panel} tab
	 */
	onTabChange: function(tabPanel, tab) {
		Sb.Editor.Ui.mainWindow.setHelperMessageHeight(0, true);

		this.helperMessage = tab.helperMessage;
		Sb.Editor.Ui.mainWindow.setHelperMessage(tab.helperMessage, true);
	},

	/**
	 * Allows us to safely switch to subtab of this tab.
	 *
	 * @param {String} subTabItemId The subtab`s itemId being activated
	 */
	setActiveTab: function(subTabItemId) {
		var subTab			= this.tabs.getComponent(subTabItemId);
		var parentTabPanel	= this.ownerCt.ownerCt.tabs;
		if (this.tabs.activeTab && !subTab.rendered) {
			this.tabs.activeTab.hide();
			parentTabPanel.setActiveTab(this);
			this.tabs.setActiveTab(subTab);
		} else {
			this.tabs.setActiveTab(subTab);
			parentTabPanel.setActiveTab(this);
		}
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function(afterSaveCallbacks) {
		if (this.showActiveErrors()) {
			afterSaveCallbacks.failure();
			return;
		}
		Sb.Editor.model.design.save({
			success: function() {
				afterSaveCallbacks.success.call();
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					SbGetLocaleByKey('messageDesignStyleSettingsSaved')
				);
			},
			failure: afterSaveCallbacks.failure
		});
	},

	/**
	 * Discard handler
	 */
	discardHandler: function() {
		Sb.Editor.model.design.restoreFromMemento(this.memento);
	},

	/**
	 * Return true if selected place such as
	 * watermark, site background, content, header e.t.c. is container for widgets
	 * @return {Boolean}
	 */
	isContainerContent: function(bgPlace) {
		bgPlace = bgPlace || this.form.findField('bgPlace').getValue();
		return !(bgPlace == 'site' || bgPlace == 'watermark');
	},

	/**
	 * update preview area
	 */
	onUpdatePreview: function() {
		var fields = {
			textColor: {
				id: 'preview-color-text',
				style: 'color',
				used: this.isContainerContent()
			},
			h1Color: {
				id: 'preview-color-h1',
				style: 'color',
				used: this.isContainerContent()
			},
			h2Color: {
				id: 'preview-color-h2',
				style: 'color',
				used: this.isContainerContent()
			},
			backgroundColor: {
				id: 'preview-color-background',
				style: 'background-color',
				used: true
			}
		};
		Ext.iterate(fields, function(item, index) {
			if (fields[item]['used']) {
				var field = this.form.findField(item);
				if (!field.isValidatedOk) {
					return;
				}
				var color = field.getValue();
				Ext.get(fields[item]['id']).setStyle(fields[item]['style'], color);
				if (item == 'backgroundColor') {
					this.form.findField('backgroundGradient').setBackgroundColor(color);
				}
			}
		}, this);
	}
});
Ext.reg('stylepanel', Sb.editor.StylePanel);
