/**
 * @class Sb.model.Node
 * @extends Ext.util.Observable
 * Base class for model nodes
 * @constructor
 * @param {Object} config The configuration options for the model element
 */
Sb.model.Node = Ext.extend(Ext.util.Observable, {

	isLoaded: false,
	isLoading: false,

	api: {
		load: undefined,
		save: undefined
	},

	constructor: function(config) {
		config = config || {};

		this.addEvents(
			/**
			 * @event load
			 * Fires when the node has been successfuly loaded and unserialized.
			 * @param {Sb.model.Node} this This node.
			 */
			'load',
			/**
			 * @event save
			 * Fires when the node has been successfully saved.
			 * @param {Sb.model.Node} this This node.
			 */
			'save',
			/**
			 * @event change
			 * Fires after the model node's data has changed.
			 * @param {Sb.model.Node} this This node.
			 */
			'change'
		);

		/**
		 * This nodes's attributes. Read-only.
		 * @type Object
		 * @property data
		 */
		this.data = {};

		Sb.model.Node.superclass.constructor.call(this);
		this.unserialize(config);
	},

	/**
	 * Updates the attributes and child of this Node from server.
	 */
	load: function() {
		if (this.isLoading) { // if an async load is already running, skipping new requests
			return;
		}

		if (this.api.load) {
			this.isLoading = true;
			this.api.load(this.loadCallback, this);
		}
	},

	/**
	 * Load callback
	 *
	 * @param {Object} result
	 * @param {Ext.Direct.Event} event
	 */
	loadCallback: function(result, event) {
		this.isLoading = false;
		if (!event.status) {
			return;
		}

		this.isLoaded = true;
		this.unserialize(result);
		this.fireEvent('load', this);
	},

	/**
	 * Fill model attributes and child models from JSON
	 *
	 * @param {Object} data May have key rootData
	 */
	unserialize: function(data) {
		var isChanged = false;

		Ext.iterate(data, function(key, value) {
			if (this.data[key] instanceof Sb.model.Node) {
				value.rootData = data.rootData;
				isChanged = this.data[key].unserialize(value) || isChanged;
				delete this.data[key].rootData;
			} else if (Ext.isObject(value)) {
				isChanged = this.unserializeObject(key, value) || isChanged;
			} else {
				isChanged = this.unserializeProperty(key, value) || isChanged;
			}
		}, this);

		if (isChanged) {
			this.fireEvent('change', this);
		}
		return isChanged;
	},

	/**
	 * Unserializes objects
	 * @param {String} key
	 * @param {Object} value
	 */
	unserializeObject: function(key, value) {
		this.data[key] = Ext.apply({}, value);
		return true;
	},

	/**
	 * Unserializes single property
	 * @param {String} key
	 * @param {String|Number|Boolean} value
	 */
	unserializeProperty: function(key, value) {
		if (this.data[key] !== value) {
			this.data[key] = value;
			this.onUnserializePropertyChange(key);
			return true;
		}
		return false;
	},

	/**
	 * Fires on changing of single property change to allow us fire up more abstract change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: Ext.emptyFn,

	/**
	 * Sets this node`s property using unserialize.
	 *
	 * @param {String} key
	 * @param {String} value
	 * @return {Boolean} Result of setting
	 */
	setProperty: function(key, value) {
		var data = {};
		data[key] = value;
		return this.unserialize(data);
	},

	restoreFromMemento: function(memento) {
		this.unserialize(memento);
	},

	/**
	 * Save model.
	 *
	 * @param {Object} callbacks Object with success and failure callbacks.
	 */
	save: function(callbacks) {
		if (this.api.save) {
			this.api.save(this.serialize(), this.saveCallback.createDelegate(this, [callbacks], true));
		}
	},

	/**
	 * Saves model if it actually changed
	 * @param {Object} mementoBefore Memento object before changing
	 * @param {Object} callbacks Object with success and failure callbacks.
	 */
	saveIfChanged: function(mementoBefore, callbacks) {
		var before	= Ext.util.JSON.encode(mementoBefore);
		var now		= Ext.util.JSON.encode(this.saveToMemento());
		if (before == now) {
			return;
		}
		this.save(callbacks);
	},

	/**
	 * Save callback
	 *
	 * @param {Object} result
	 * @param {Ext.Direct.Event} event
	 * @param {Object} callbacks
	 */
	saveCallback: function(result, event, callbacks) {
		callbacks = callbacks || {};
		if (event.status) {
			this.fireEvent('save', this);
			if (callbacks.success) {
				callbacks.success();
			}
		} else {
			if (callbacks.failure) {
				callbacks.failure();
			}
		}
	},

	/**
	 * Create JSON from model attributes and child models
	 *
	 * @param {Object} rootData Data of object, that intiates serialize. Should be passed in each child serilize.
	 * @return {Object}
	 */
	serialize: function(rootData) {
		var data = {};
		Ext.iterate(this.data, function(key, value) {
			if (value instanceof Sb.model.Node) {
				data[key] = value.serialize(rootData);
			} else if (Ext.isObject(value)) {
				data[key] = Ext.apply({}, value);
			} else {
				data[key] = value;
			}
		}, this);
		return data;
	},

	saveToMemento: function() {
		return this.serialize();
	},

	/**
	 * Destroys a model, calling common onDestroy
	 */
	destroy: function() {
		this.onDestroy();
	},

	onDestroy: Ext.emptyFn
});
