/**
 * @class Sb.model.Site
 * @extends Sb.model.Node
 * Class for site model
 * @constructor
 * @param {Object} config The configuration options for the site model
 */
Sb.model.Site = Ext.extend(Sb.model.Node, {
	constructor: function(config) {
		this.addEvents(
			/**
			 * @event beforeload
			 * Fires after node has been successfuly loaded and before serialized.
			 * @param {Sb.model.Node} this This node.
			 */
			'beforeload'
		);

		this.api = {
			load: Sb.SiteService.getSettings,
			save: Sb.SiteService.setSettings
		};

		/**
		 * The model node of site design
		 * @type Sb.model.SiteDesign
		 * @property design
		 */
		this.design = new Sb.model.SiteDesign();

		/**
		 * The model node of site root page
		 * @type Sb.model.SitePage
		 * @property rootPage
		 */
		this.rootPage = new Sb.model.SitePage();

		/**
		 * Images, that is used in site and uploaded by user
		 *
		 * @type Ext.util.MixedCollection
		 * @property usedImages
		 */
		this.uploadedImages = new Ext.util.MixedCollection();

		Sb.model.Widget.superclass.constructor.call(this, config);
	},

	/**
	 * Load callback
	 *
	 * @param {Object} result
	 * @param {Ext.Direct.Event} event
	 */
	loadCallback: function(result, event) {
		this.isLoading = false;
		if (!event.status) {
			return;
		}

		this.isLoaded = true;
		this.fireEvent('beforeload', this);
		(function() {
			this.unserialize(result);
			this.fireEvent('load', this);
		}).defer(10, this);
	},

	unserialize: function(data) {
		var isChanged = false;

		if (Ext.isObject(data.design)) {
			data.design.rootData = data;
			isChanged = this.design.unserialize(data.design) || isChanged;
			delete this.design.data.rootData;
			delete data.design;
		}

		if (Ext.isObject(data.rootPage)) {
			data.rootPage.rootData = data;
			isChanged = this.rootPage.unserialize(data.rootPage) || isChanged;
			delete data.rootPage;
		}

		return Sb.model.Site.superclass.unserialize.call(this, data) || isChanged;
	},

	serialize: function() {
		var data = Sb.model.Site.superclass.serialize.call(this, {});
		data.design = this.design.serialize(data);
		data.rootPage = this.rootPage.serialize(data);
		return data;
	},

	/**
	 * Gets the site url.
	 * @return {String}
	 */
	getUrl: function() {
		return Sb.Config.sitesUrl + '/' + this.data.dirName.substr(0, 2) + '/' + this.data.dirName;
	},

	/**
	 * Save model.
	 *
	 * @param {Object} callbacks Object with success and failure callbacks.
	 */
	save: function(callbacks) {
		var settings = {
			siteName			: this.data.title,
			pagesUseName		: this.data.pagesUseName,
			description			: this.data.description,
			keywords			: this.data.keywords,
			customHead			: this.data.customHead,
			googleAnalyticsCode	: this.data.googleAnalyticsCode,
			faviconFile			: this.data.faviconFile
		};
		this.api.save(settings, this.saveCallback.createDelegate(this, [callbacks], true));
	},

	/**
	 * Updates all used uploaded images in site
	 *
	 * @return {Ext.util.MixedCollection} updated images collection
	 */
	updateUploadedImages: function() {
		var reIsAttachmentImage		= new RegExp(
			Sb.Config.imagePath + '/([^/]+\\.(' + Sb.Config.imageValidExtensions.join('|') + '))"?', 'i'
		);
		var reIsAttachmentImages	= new RegExp(reIsAttachmentImage.source, 'ig');
		var site					= this;
		site.uploadedImages.clear();
		site.rootPage.applySelfAndRecursive(function (page) {
			page.widget.widgets.each(function(verticalContainer) {
				verticalContainer.widgets.each(function(widget) {
					if (!widget.data.content) {
						return;
					}
					var matches = widget.data.content.match(reIsAttachmentImages);
					if (null != matches) {
						Ext.each(matches, function(usedImage) {
							usedImage = decodeURIComponent(usedImage.match(reIsAttachmentImage)[1]);
							if (!site.uploadedImages.containsKey(usedImage)) {
								site.uploadedImages.add(usedImage, 0);
							}
							site.uploadedImages.add(usedImage, site.uploadedImages.get(usedImage) + 1);
						});
					}
				});
			});
		});
		return site.uploadedImages;
	}
});
