/**
 * @class Sb.model.SiteDesign
 * @extends Sb.model.Node
 * Class for site design model
 * @constructor
 * @param {Object} config The configuration options for the site design model
 */
Sb.model.SiteDesign = Ext.extend(Sb.model.Node, {
	constructor: function(config) {
		this.api = {
			load: Sb.SiteService.getDesignSettings,
			save: Sb.SiteService.setDesignSettings
		};

		this.addEvents(
			/**
			 * @event decorationchange
			 * Fires after the decoration has changed.
			 * @param {String} decorationType
			 * @param {String} decoration Name of decoration
			 */
			'decorationchange',
			/**
			 * @event colroschemechange
			 * Fires after the color scheme has changed.
			 */
			'colorschemechange',
			/**
			 * @event layoutchange
			 * Fires after the layout has changed.
			 */
			'layoutchange',
			/**
			 * @event fontchange
			 * Fires after the font has changed.
			 */
			'fontchange',
			/**
			 * @event backgroundcolorchange
			 * Fires after the background colors has changed.
			 */
			'backgroundcolorchange'
		);

		Sb.model.SiteDesign.superclass.constructor.call(this, config);

		/**
		 * Scheme colors. Read-only.
		 * @type Object
		 * @property palette
		 */
		this.palette = {};

		/**
		 * Custom colors. Read-only.
		 * @type Object
		 * @property customColors
		 */
		this.customColors = {};
	},

	/**
	 * Return design settings
	 *
	 * @return Sb.model.SiteDesign.data
	 */
	getSettings: function() {
		return this.data;
	},

	/**
	 * Return list of used decorators by decoration type
	 *
	 * @param {String} decorationType For example: background or border
	 * @return {Array} Like ['backgrounds1', 'backgrounds2']
	 */
	getDecoratorList: function(decorationType) {
		var list = [];
		Ext.iterate(this.data[decorationType], function(property, decoration) {
			if (decoration && decoration.length > 1 && list.indexOf(decoration) == -1) {
				list.push(decoration);
			}
		}, this);
		return list;
	},

	/**
	 * Apply new decoration
	 *
	 * @param {String} decorationType Decoration type
	 * @param {String} property Decoration name
	 * @param {String} decoration Decoration value
	 */
	setDecoration: function(decorationType, property, decoration) {
		var properties = {};
		properties[property] = decoration;

		return this.unserializeObject(decorationType, properties);
	},

	/**
	 * Unserializes objects
	 * @param {String} key
	 * @param {Object} value
	 */
	unserializeObject: function(category, properties) {
		var isChanged = false;
		var decorationValue = null;

		this.data[category] = this.data[category] || {};
		
		Ext.iterate(properties, function(property, value) {
			if (this.palette[property]) {
				if (value == '' || value == this.palette[property]) {
					value = this.palette[property];
					this.customColors[property] = false;
				} else {
					this.customColors[property] = true;
				}
			}
			if (this.data[category][property] != value) {
				this.data[category][property] = value;
				decorationValue = value;
				isChanged = true;
			}
		}, this);

		if (isChanged) {
			if (category == 'colorScheme') {
				this.updatePalette();
			}
			if (category == 'background' || category == 'border' || category == 'external-border') {
				this.fireEvent('decorationchange', category, decorationValue);
			} else {
				this.fireEvent(category.toLowerCase() + 'change', this);
			}
		}

		return isChanged;
	},

	/**
	 * Update palette colors and reset all custom colors.
	 */
	updatePalette: function() {
		var scheme = this.data.colorScheme;
		// check light colors

		this.palette = {
			// site colors
			siteColor		: scheme.color1,
			watermarkColor	: scheme.color2
		};

		var containerPalettes = {
			header	: {color1: scheme.color3, color2: scheme.color1, color3: scheme.color4},
			footer	: {color1: scheme.color3, color2: scheme.color1, color3: scheme.color4},
			column1	: {color1: scheme.color4, color2: scheme.color1, color3: scheme.color2},
			column2	: {color1: scheme.color4, color2: scheme.color1, color3: scheme.color2},
			content	: {color1: scheme.color2, color2: scheme.color3, color3: scheme.color4}
		};

		Ext.iterate(containerPalettes, function(region, scheme) {
			this.palette[region + 'Color'] = scheme.color1;

			Ext.iterate(this.generateTextPalette(scheme), function(property, value) {
				this.palette[region + property] = value;
			}, this);

			Ext.iterate(this.generateNavigationPalette(scheme), function(property, value) {
				this.palette[region + property] = value;
			}, this);
		}, this);

		this.resetCustomColors();
	},

	/**
	 * Generate colors for text widget.
	 * @param {Object} scheme Base colors for generation
	 * @return {Object}
	 */
	generateTextPalette: function(scheme) {
		var isLight = {};
		Ext.iterate(scheme, function(name, value) {
			isLight[name] = Ext.ux.util.Color.isLight(value);
		});

		return {
			TextColor	: isLight.color1 ? '#000000' : '#FFFFFF',
			LinkColor	: isLight.color1 ? '#0000FF' : '#33FFFF',
			H1Color		: Ext.ux.util.Color.getVariant(scheme.color2, 0, isLight.color1 ? -0.7 : 0.7),
			H2Color		: Ext.ux.util.Color.getVariant(scheme.color2, 0, isLight.color1 ? -0.8 : 0.8)
		};
	},

	/**
	 * Generate colors for navigation widget.
	 * @param {Object} scheme Base colors for generation
	 * @return {Object}
	 */
	generateNavigationPalette: function(scheme) {
		var isLight = {};
		Ext.iterate(scheme, function(name, value) {
			isLight[name] = Ext.ux.util.Color.isLight(value);
		});

		return {
			NavigationLevel1ContainerBackgroundColor	: 'transparent',
			NavigationLevel1Color						: isLight.color1 ? '#000000' : '#FFFFFF',
			NavigationLevel1BackgroundColor				: 'transparent',
			NavigationLevel1ColorSelected				: isLight.color2 ? '#000000' : '#FFFFFF',
			NavigationLevel1BackgroundColorSelected		: scheme.color2,
			NavigationLevel1ColorHovered				: isLight.color3 ? '#000000' : '#FFFFFF',
			NavigationLevel1BackgroundColorHovered		: scheme.color3,
			NavigationLevel2ContainerBackgroundColor	: scheme.color2,
			NavigationLevel2Color						: isLight.color2 ? '#000000' : '#FFFFFF',
			NavigationLevel2BackgroundColor				: scheme.color2,
			NavigationLevel2ColorSelected				: isLight.color2 ? '#000000' : '#FFFFFF',
			NavigationLevel2BackgroundColorSelected		: scheme.color2,
			NavigationLevel2ColorHovered				: isLight.color3 ? '#000000' : '#FFFFFF',
			NavigationLevel2BackgroundColorHovered		: scheme.color3
		};
	},

	/**
	 * Reset all custom colors.
	 */
	resetCustomColors: function() {
		Ext.iterate(this.data, function(category, properties) {
			var newProperties = {};
			Ext.iterate(properties, function(property, value) {
				if (this.palette[property]) {
					newProperties[property] = '';
				}
			}, this);
			this.unserializeObject(category, newProperties);
		}, this);
	}
});
