/**
 * @class Sb.model.Widget
 * @extends Sb.model.Node
 * Class for widget model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Sb.model.Widget = Ext.extend(Sb.model.Node, {

	/**
	 * @cfg {String} uuid
	 * The widget uuid (defaults to new generated uuid)
	 */

	/**
	 * @cfg {String} type
	 * text, navigation etc. (defaults to <tt>text</tt>)
	 */

	constructor: function(config) {
		this.api = {
			save: Sb.WidgetService.save
		};

		config = Ext.apply({
			type: 'text',
			uuid: Sb.Utils.generateUuid()
		}, config);

		/**
		 * The collection of child widgets as a {@link Ext.util.MixedCollection}
		 * @type MixedCollection
		 * @property widgets
		 */
		this.widgets = new Ext.util.MixedCollection(false, function(childWidget) {
			if (childWidget instanceof Sb.model.ContainerVerticalWidget) {
				return childWidget.data.region;
			}
			if (childWidget instanceof Sb.model.Widget) {
				return childWidget.data.type + childWidget.data.uuid;
			}
		});

		Sb.model.Widget.superclass.constructor.call(this, config);
	},

	unserialize: function(data) {
		var isChanged = false;

		if (undefined != data.widgetList) {
			data.rootData = {widgetList: data.widgetList};
			delete data.widgetList;
		}

		if (Ext.isArray(data.widgets)) {
			Ext.each(data.widgets, function(widgetData) {
				if (Ext.isString(widgetData) && data.rootData != undefined && data.rootData.widgetList != undefined) {
					widgetData = data.rootData.widgetList[widgetData];
					if (widgetData) {
						widgetData.rootData = data.rootData;
					}
				}
				var widget = Sb.WidgetTypeManager.create(widgetData);
				delete widget.data.rootData;
				this.widgets.add(widget);
			}, this);
			delete data.widgets;
			isChanged = true;
		}

		return Sb.model.Widget.superclass.unserialize.call(this, data) || isChanged;
	},

	serialize: function(rootData) {
		var isInitiator	= !rootData; rootData = rootData || {};
		var data		= Sb.model.Widget.superclass.serialize.call(this, rootData);
		var passData	= (isInitiator) ? data : rootData;

		if (!rootData['widgetList']) {
			rootData['widgetList'] = {};
		}
		if (!rootData['widgetList'][this.data.uuid]) {
			data.widgets = [];
			this.widgets.each(function(widget) {
				data.widgets.push(widget.serialize(passData));
			});
			rootData['widgetList'][this.data.uuid] = data;
		}
		return (isInitiator) ? data : this.data.uuid;
	},

	/**
	 * Create child widget.
	 * @param {Sb.widgetType.Base} type Type of the new widget
	 * @param {Number} index The index to insert the item at.
	 */
	createChild: function(type, index) {
		var mockWidget = Sb.WidgetTypeManager.create({
			type: type.getPhpType(),
			uuid: 'mockWidget'
		});
		Sb.WidgetService.createChild(this.data.uuid, type.getPhpType(), index, function(result, response) {
			Sb.WidgetTypeManager.remove(mockWidget);
			if (!response.status) {
				return;
			}
			var widget = Sb.WidgetTypeManager.create(result);
			widget.isNew = true;
			if (Ext.isNumber(index)) {
				this.widgets.insert(index, widget);
			} else {
				this.widgets.add(widget);
			}
		}, this);
	},

	/**
	 * Remove child widget.
	 * @param {Sb.model.Widget} object Removes widget from model.
	 * @param {Boolean} isSuppressEvents Do we suspend events on widgets collections in callback? Default false.
	 */
	removeChild: function(object, isSuppressEvents) {
		var index = this.widgets.indexOf(object);
		if (index == -1) {
			return;
		}
		if (isSuppressEvents) {
			this.widgets.suspendEvents();
		}
		this.widgets.removeAt(index);
		Sb.WidgetTypeManager.remove(object);
		object.destroy();
		if (isSuppressEvents) {
			this.widgets.resumeEvents();
		}
		Sb.WidgetService.removeChild(this.data.uuid, index);
	},

	/**
	 * Insert widget.
	 * @param {Number} index The index to insert the item at.
	 * @param {Sb.model.Widget} widget Widget for inserting.
	 * @param {Sb.model.Widget} widget Current widget container.
	 */
	insert: function(index, widget, container) {
		if (index == undefined) {
			return;
		}
		var sourceIndex = container.widgets.indexOf(widget);
		if (sourceIndex == -1) {
			return;
		}
		container.widgets.remove(widget);
		this.widgets.insert(index, widget);
		Sb.WidgetService.insertChild(this.data.uuid, index, container.data.uuid, sourceIndex);
	},

	/**
	 * Copy child widget to this widget.
	 *
	 * @param {Number} index The index to copy item at.
	 * @param {Sb.model.Widget} widget Widget for copy.
	 * @param {Boolean} isSuppressEvents Do we suspend events on widgets collections in callback? Default false.
	 */
	copyChild: function(index, widget, isSuppressEvents) {
		if (index == undefined) {
			index = 0;
		}
		if (isSuppressEvents) {
			this.widgets.suspendEvents();
		}
		this.widgets.insert(index, widget);
		Sb.WidgetTypeManager.restore(widget);
		if (isSuppressEvents) {
			this.widgets.resumeEvents();
		}
		Sb.WidgetService.copyChild(this.data.uuid, widget.data.uuid, index);
	}
});