/**
 * @class Sb.widget.ContainerFixed
 * @extends Ext.Container
 * @constructor
 * @param {Object} config The config object
 * @xtype container_fixed_widget
 */
Sb.widget.ContainerFixed = Ext.extend(Ext.Container, {
	/**
	 * @cfg {Sb.model.Widget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	/**
	 * object already loaded borders 
	 */
	loadedBorders: {},

	initComponent : function() {
		this.items = [];
		this.model.widgets.each(function(widget) {
			this.items.push({
				xtype : widget.data.type + '_widget',
				model : widget
			});
		}, this);
		this.settedBorder = '';
		Sb.widget.ContainerFixed.superclass.initComponent.call(this);
	},

	onRender : function(ct, position) {
		var siteData = Sb.Editor.model.data;

		var partial = new Sb.PartialTemplate(Sb.Editor.model.data.template.partials['page']);
		var markup = partial.apply({}).trim();

		markup = markup.replace(/<Sitebuilder\:(\w+)((\s+(\w+)\s*=\s*"([^"]*)")*)\s*\/>/g, function(){
			var attrs = {};
			Ext.each(arguments[2].match(/(\w+)\s*=\s*"([^"]*)"/g), function(matches) {
				var m = matches.match(/(\w+)\s*=\s*"([^"]*)"/);
				attrs[m[1]] = m[2];
			});
			return '<div id="sb-container-' + attrs.id + '"></div>';
		});
		markup = markup.replace(/images\/s\.gif/g, Sb.Config.blankImageUrl);

		this.el = ct.insertHtml('afterBegin', markup, true);
		this.watermarkEl = this.el.down('#watermark');
		this.columnsEl = this.watermarkEl.down('#columns');
	},

	afterRender : function() {
		this.items.each(function(item) {
			var placeholder = Ext.fly('sb-container-' + item.model.data.region);
			item.render(placeholder);
			item.el.insertAfter(placeholder);
			placeholder.remove();
			this[item.model.data.region] = item;
		}, this);

		this.onDecorationChange();
		this.onBackgroundColorChange();
		this.onLayoutChange();

		this.mon(Sb.Editor.model.design, {
			scope: this,
			decorationchange: this.onDecorationChange,
			backgroundcolorchange: this.onBackgroundColorChange,
			layoutchange: this.onLayoutChange
		});
		this.mon(this.watermarkEl, 'contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
			type: 'watermark'
		});
	},

	onDecorationChange: function() {
		this.setSiteBackground();
		this.setWatermarkBackground();
		this.setWatermarkExternalBorder();
		this.setWatermarkBorder();
	},

	/**
	 * Update styles when layout was changed
	 */
	onLayoutChange: function() {
		var data = Sb.Editor.model.design.data.layout;
		var contentLeftMargin = data.sidebarFirst
			? parseInt(data.sidebarFirstWidth) + parseInt(data.horizontalMargin)
			: 0;
		var contentRightMargin = data.sidebarSecond
			? parseInt(data.sidebarSecondWidth) + parseInt(data.horizontalMargin)
			: 0;
		var minWidth = contentLeftMargin + contentRightMargin + 100;

		this.el.setStyle({
			'min-width': minWidth + 'px',
			'width': data.width + (data.type == 'elastic' ? '%' : 'px')
		});

		this.columnsEl.setStyle({
			'padding': data.verticalMargin + 'px ' + data.horizontalMargin + 'px'
		});

		this.content.el.setStyle({
			'margin-left': (data.sidebarFirstPosition == 'left' ? contentLeftMargin : contentRightMargin) + 'px',
			'margin-right': (data.sidebarFirstPosition == 'right' ? contentLeftMargin : contentRightMargin) + 'px'
		});
		this.content.contentElement.parent().setStyle({
			'min-height': data.contentMinHeight + 'px'
		});

		if (!this.column1.el.hasClass('column-' + data.sidebarFirstPosition)) {
			this.column1.el.addClass('column-' + data.sidebarFirstPosition);
			this.column1.el.removeClass('column-' + (data.sidebarFirstPosition == 'left' ? 'right' : 'left'));
		}

		if (!this.column2.el.hasClass('column-' + data.sidebarSecondPosition)) {
			this.column2.el.addClass('column-' + data.sidebarSecondPosition);
			this.column2.el.removeClass('column-' + (data.sidebarSecondPosition == 'left' ? 'right' : 'left'));
		}

		this.column1.el.setStyle({
			'display': data.sidebarFirst ? 'block' : 'none',
			'width': data.sidebarFirstWidth + 'px'
		});
		this.column1.contentElement.parent().setStyle({
			'min-height': data.sidebarFirstMinHeight + 'px'
		});
		this.column2.el.setStyle({
			'display': data.sidebarSecond ? 'block' : 'none',
			'width': data.sidebarSecondWidth + 'px'
		});
		this.column2.contentElement.parent().setStyle({
			'min-height': data.sidebarSecondMinHeight + 'px'
		});
	},

	/**
	 * Update background color.
	 */
	onBackgroundColorChange: function() {
		this.watermarkEl.setStyle(
			'background-color',
			Sb.Editor.model.design.data.backgroundColor.watermarkColor
		);
	},

	setSiteBackground: function() {
		if (Sb.Editor.model.design.data.background.siteBackground) {
			Ext.getBody().dom.className = this._addDecorationToClassName(
				Ext.getBody().dom.className,
				'background',
				Sb.Editor.model.design.data.background.siteBackground
			).join(' ');
		}
	},

	setWatermarkBackground: function() {
		var background = Sb.Editor.model.design.data.background.watermarkBackground;

		if (background) {
			this.watermarkEl.dom.className = this._addDecorationToClassName(
				this.watermarkEl.dom.className,
				'background',
				background
			).join(' ');

			Ext.get('watermark-background-image').dom.src = Sb.Config.templatesUrl + '/'
				+ Sb.Editor.model.data.template.code
				+ '/decorations/background/' + background + '/images/' + background + '.png';
		}
	},

	/**
	 * apply new styles for external-borders
	 * @param {String} path
	 * @param {Object} result 
	 * @param {String} border
	 */
	setStylesForBorder: function(path, result, border) {
		//external-top
		Ext.get('wrapper').down('div.external-top').setStyle({
			'padding': '0 0 0 ' + result.width + 'px',
			'background-image': 'url(\'' + path +  '/images/' + border + '-top-left.png\')'
		}).down('div').setStyle({
			'padding': '0 ' + result.width + 'px 0 0',
			'background-image': 'url(\'' + path +  '/images/' + border + '-top-right.png\')'
		}).down('div').setStyle({
			'padding': '0 0 0 ' + result.decorationWidth + 'px',
			'background-image': 'url(\'' + path +  '/images/' + border + '-top-left2.png\')'
		}).down('div').setStyle({
			'padding': '0 ' + result.decorationWidth + 'px 0 0',
			'background-image': 'url(\'' + path +  '/images/' + border + '-top-right2.png\')'
		}).down('div').setStyle({
			'height': result.height + 'px',
			'background-image': 'url(\'' + path +  '/images/' + border + '-top.png\')'
		});

		//external-bottom
		Ext.get('wrapper').down('div.external-bottom').setStyle({
			padding: '0 0 0 ' + result.width + 'px',
			'background-image': 'url(\'' + path +  '/images/' + border + '-bottom-left.png\')' 
		}).down('div').setStyle({
			padding: '0 ' + result.width + 'px 0 0',
			'background-image': 'url(\'' + path +  '/images/' + border + '-bottom-right.png\')' 
		}).down('div').setStyle({
			'padding': '0 0 0 ' + result.decorationWidth + 'px',
			'background-image': 'url(\'' + path +  '/images/' + border + '-bottom-left2.png\')' 
		}).down('div').setStyle({
			'padding': '0 ' + result.decorationWidth + 'px 0 0',
			'background-image': 'url(\'' + path +  '/images/' + border + '-bottom-right2.png\')' 
		}).down('div').setStyle({
			height: result.height + 'px',
			'background-image': 'url(\'' + path +  '/images/' + border + '-bottom.png\')' 
		});

		//external-side-left
		Ext.get('wrapper').down('div.external-side-left').setStyle({
			'background-image': 'url(\'' + path +  '/images/' + border + '-left.png\')'
		}).down('div').setStyle({
			'background-image': 'url(\'' + path +  '/images/' + border + '-left-top.png\')'
		}).down('div').setStyle({
			'background-image': 'url(\'' + path +  '/images/' + border + '-left-bottom.png\')'
		}).down('div').setStyle({//external-side-right
			'background-image': 'url(\'' + path +  '/images/' + border + '-right.png\')'
		}).down('div').setStyle({
			'background-image': 'url(\'' + path +  '/images/' + border + '-right-top.png\')'
		}).down('div').setStyle({
			'background-image': 'url(\'' + path +  '/images/' + border + '-right-bottom.png\')',
			'padding': '0 ' + result.width + 'px 0 ' + result.width + 'px' 
		});
	},

	setWatermarkExternalBorder: function() {
		var border = Sb.Editor.model.design.data['external-border'].watermarkExternalBorder;
		if (border != this.settedBorder) {
			this.settedBorder = border;
			var cacheKey = border.replace(/-/g, '', border);
			var path = Sb.Config.templatesUrl + '/' +
				encodeURIComponent(Sb.Editor.model.data.template.code) + 
				'/decorations/external-border/' + encodeURIComponent(border);
			if (!this.loadedBorders[cacheKey]) {
				Sb.DesignTemplateService.getDecorationInfo('generic', 'external-border', border, function(result) {
					this.loadedBorders[cacheKey] = result;
					this.setStylesForBorder(path, this.loadedBorders[cacheKey], border);
				}.createDelegate(this));
			} else {
				this.setStylesForBorder(path, this.loadedBorders[cacheKey], border);
			}
		}
	},

	setWatermarkBorder: function() {
		var border = Sb.Editor.model.design.data.border.watermarkBorder;
		if (border) {
			this.watermarkEl.dom.className = this._addDecorationToClassName(
				this.watermarkEl.dom.className,
				'border',
				border
			).join(' ');
		}
	},

	/**
	 * Add or replace css class of decoration to element classes
	 *
	 * @param {String} className Css classes of element
	 * @param {String} decorationType For example: background or border
	 * @param {String} decoration Decoration value
	 *
	 * @return {Array} Css classes of element
	 */
	_addDecorationToClassName: function(className, decorationType, decoration) {
		var classes = className.split(' ');
		Ext.each(classes, function(cssClass, index) {
			if (cssClass.indexOf(decorationType) == 0 || cssClass == '') {
				delete classes[index];
			}
		}, this);
		classes.push(decoration);
		if (decorationType == 'background') {
			classes.push('background');
		}
		return classes;
	}
});

Ext.reg('container_fixed_widget', Sb.widget.ContainerFixed);
