/**
 * @class Sb.widget.Navigation
 * @extends Sb.widget.Widget
 * Class for Navigation widget
 * @constructor
 * @param {Object} config The config object
 * @xtype navigation_widget
 */
Sb.widget.Navigation = Ext.extend(Sb.widget.Widget, {

	cls: 'sb-widget-navigation',

	/**
	 * @cfg {Sb.model.NavigationWidget} model
	 * The {@link Sb.model.NavigationWidget} to bind this Widget to.
	 */

	onRender: function() {
		Sb.widget.Navigation.superclass.onRender.apply(this, arguments);

		this.innerEl.addClass('clearfix');

		this.menu.add({
			text: SbGetLocaleByKey('cmPages'),
			iconCls: 'sb-pages-icon',
			scope: this,
			handler: function() {
				this.completeEdit();
				Sb.Editor.Ui.mainWindow.tabs.setActiveTab('pages');
			}
		});
		this.menu.add({
			text: SbGetLocaleByKey('cmWidgetNavigationItems'),
			iconCls: 'sb-menu-items-style-icon',
			scope: this,
			handler: function() {
				this.startEdit();
				Sb.widget.Navigation.editPanel.tabs.setActiveTab(1); // Items panel
			}
		});

		this.navigationEl = this.contentElement.createChild({
			tag: 'ul',
			id:  'navigation-' + this.model.data.uuid,
			cls: 'navigation'
		});
		this.contentElement.createChild({
			cls: 'cleared',
			html: '<!-- clear -->'
		});
	},

	afterRender: function() {
		Sb.widget.Navigation.superclass.afterRender.call(this);
		this.update();
		this.onFontColorAlignmentChange.call(this);
	},

	getContentTarget: function() {
		return this.navigationEl;
	},

	initEvents: function() {
		Sb.widget.Navigation.superclass.initEvents.call(this);

		this.model.on('change', this.update, this);
		this.model.on('levelchange', this.update, this);
		this.model.on('fontchange', this.onFontColorAlignmentChange, this);
		this.model.on('colorchange', this.onFontColorAlignmentChange, this);
		this.model.on('alignmentchange', this.onFontColorAlignmentChange, this);
		this.model.on('save', function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(
				SbGetLocaleByKey('messageChangesSaved')
			);
		});
		var navigationWidget = this;
		Sb.Editor.model.rootPage.applyRecursive(function (page) {
			this.pages.on('add', navigationWidget.update, navigationWidget);
			this.pages.on('remove', navigationWidget.update, navigationWidget);
		});

		if (Ext.isIE7) {
			this.getEl().on('mouseenter', function() {
				var parentNode = Ext.get(this.el.findParentNode('#header-content'));
				if (!Sb.Editor.Ui.isDragActive() && !Sb.widget.Widget.editCurrent && parentNode) {
					parentNode.setStyle('z-index', this.el.getStyle('z-index'));
				}
			}, this);
			this.getEl().on('mouseleave', function() {
				var parentNode = Ext.get(this.el.findParentNode('#header-content'));
				if (!Sb.Editor.Ui.isDragActive() && !Sb.widget.Widget.editCurrent && parentNode) {
					parentNode.setStyle('z-index', 0);
				}
			}, this);
		}
	},

	/**
	 * Remove "over" state
	 */
	removeOver: function() {
		Sb.widget.Navigation.superclass.removeOver.call(this);
		if (!Sb.Editor.Ui.isDragActive() && Sb.widget.Widget.editCurrent != this) {
			var parentNode = Ext.get(this.el.findParentNode('#header-content'));
			if (parentNode) {
				Ext.select('#columns .sb-widget-navigation').each(function () {
					var item = Ext.getCmp(this.id);
					item.manager.sendToBack(item);
				});
			}
		}
	},

	/**
	 * Gets root page for which menu is generated.
	 *
	 * @return {Sb.model.SitePage}
	 */
	getMenuRootPage: function() {
		var rootPageIdentity	= Sb.Editor.model.rootPage.data.identity;
		var currentPageIdentity	= Sb.Editor.siteView.currentPage.data.identity;
		var menuRootPage = Sb.Editor.model.rootPage;
		if (this.model.data.level == 2) {
			Sb.Editor.model.rootPage.applyRecursive(function (page) {
				if (currentPageIdentity == page.data.identity) {
					if (rootPageIdentity == this.data.identity) {
						menuRootPage = page;
					} else {
						menuRootPage = this;
					}
				}
			});
		}
		return menuRootPage;
	},

	update: function() {
		this.cleanHandlers();

		this.navigationEl.update('');

		var currentPageIdentity = Sb.Editor.siteView.currentPage.data.identity;
		this.getMenuRootPage().pages.each(function(page) {
			var fn;
			this.handlers.push({object: page.pages, eventName: 'add', fn: this.update, scope: this});
			page.pages.on('add', this.update, this);
			this.handlers.push({object: page.pages, eventName: 'remove', fn: this.update, scope: this});
			page.pages.on('remove', this.update, this);
			var itemEl = this.navigationEl.createChild({
				tag : 'li',
				cls : (currentPageIdentity == page.data.identity) ? 'selected' : '',
				cn  : {
					tag  : 'a',
					href : '#' + escape(page.getFileName()),
					html : Ext.util.Format.htmlEncode(page.data.title)
				}
			});
			itemEl.setVisibilityMode(Ext.Element.DISPLAY);
			itemEl.setVisible(!page.data.hidden);
			fn = function(event, el, options) {
				event.stopEvent();
				Sb.Editor.siteView.setCurrentPage(options.page);
			};
			itemEl.on('click', fn, null, {delegate: 'a', 'page' : page});
			this.handlers.push({object: itemEl, eventName: 'click', fn: fn});
			fn = function() {
				itemEl.first().update(Ext.util.Format.htmlEncode(page.data.title));
				itemEl.setVisible(!page.data.hidden);
			};
			page.on('change', fn);
			this.handlers.push({object: page, eventName: 'change', fn: fn});
			page.pages.each(function(subPage) {
				if (currentPageIdentity == subPage.data.identity) {
					itemEl.addClass('childselected');
				}
			});
			if (this.model.data.depth > 1 && page.pages.getCount() > 0) {
				var submenuEl = itemEl.createChild({
					tag : 'ul'
				});
				page.pages.each(function(subPage) {
					var subitemEl = submenuEl.createChild({
						tag : 'li',
						cls : (currentPageIdentity == subPage.data.identity) ? 'selected' : '',
						cn  : {
							tag  : 'a',
							href : '#' + escape(subPage.getFileName()),
							html : Ext.util.Format.htmlEncode(subPage.data.title)
						}
					});
					subitemEl.setVisibilityMode(Ext.Element.DISPLAY);
					subitemEl.setVisible(!subPage.data.hidden);
					fn = function(event, el, options) {
						event.stopEvent();
						Sb.Editor.siteView.setCurrentPage(options.page);
					};
					subitemEl.on('click', fn, null, {delegate: 'a', 'page' : subPage});
					this.handlers.push({object: subitemEl, eventName: 'click', fn: fn});
					fn = function() {
						subitemEl.first().update(Ext.util.Format.htmlEncode(subPage.data.title));
						subitemEl.setVisible(!subPage.data.hidden);
					};
					subPage.on('change', fn);
					this.handlers.push({object: subPage, eventName: 'change', fn: fn});
				}, this);
			}
		}, this);
	},

	cleanHandlers : function() {
		if (this.handlers) {
			Ext.each(this.handlers, function(handler, index) {
				if (handler.object) {
					handler.object.un(handler.eventName, handler.fn, handler.scope);
				}
			}, this);
		}
		this.handlers = [];
	},

	onStartEdit: function() {
		this.memento = this.model.saveToMemento();
		if (!Sb.widget.Navigation.editPanel) {
			Sb.widget.Navigation.editPanel = new Sb.widget.NavigationEditPanel();
		}
		Sb.widget.Navigation.editPanel.show(this);
	},

	onCompleteEdit: function() {
		this.model.saveIfChanged(this.memento);
		if (Ext.isIE7) {
			var parentNode = Ext.get(this.el.findParentNode('#header-content'));
			if (parentNode) {
				parentNode.setStyle('z-index', 0);
			}
		}
	},

	onCancelEdit: function() {
		this.model.restoreFromMemento(this.memento);
	},

	onHideEdit: function() {
		Sb.widget.Navigation.editPanel.hide();
	},

	onDestroy : function(){
		if (this.rendered) {
			this.model.un('change', this.update, this);
			this.model.un('levelchange', this.update, this);
			this.model.un('fontchange', this.onFontColorAlignmentChange, this);
			this.model.un('colorchange', this.onFontColorAlignmentChange, this);
			this.model.un('alignmentchange', this.onFontColorAlignmentChange, this);
			var navigationWidget = this;
			Sb.Editor.model.rootPage.applyRecursive(function (page) {
				this.pages.un('add', navigationWidget.update, navigationWidget);
				this.pages.un('remove', navigationWidget.update, navigationWidget);
			});

			this.cleanHandlers();
		}
		Sb.widget.Navigation.superclass.onDestroy.call(this);
	},

	onDocumentClick: function(event) {
		if (event.getTarget('.x-combo-list') || event.getTarget('.sb-color-dropdown')) {
			return;
		}

		Sb.widget.Navigation.superclass.onDocumentClick.call(this, event);
	},

	/**
	 * Updates CSS for navigation widget.
	 */
	onFontColorAlignmentChange: function() {
		var id		= 'navigation-' + this.model.data.uuid;
		var cssName	= 'style-for-' + id;
		var data	= Ext.apply({className: '#' + id}, this.model.data);
		Ext.iterate(data, function (key, value, hash) {
			var intValue = parseInt(value);
			if (Ext.isNumber(intValue)) {
				hash[key] = intValue;
			}
		});
		data['level1LineSize']	= data['level1FontSize'] + 10;
		data['level2LineSize']	= data['level2FontSize'] + 10;
		data['alignment']		= data['level1Alignment'].toLowerCase();
		Ext.util.CSS.removeStyleSheet(cssName);
		Ext.util.CSS.createStyleSheet(
			Sb.widget.Navigation.getCssTemplate().apply(data) +
				Sb.widget.Navigation.getColorSchemeCssTemplate().apply(data),
			cssName
		);
	}
});

Ext.reg('navigation_widget', Sb.widget.Navigation);

/**
 * CSS template of navigation widget.
 * @return {Ext.XTemplate} this
 */
Sb.widget.Navigation.getCssTemplate = function() {
	if (!Sb.widget.Navigation.cssTemplate) {
		Sb.widget.Navigation.cssTemplate = new Ext.XTemplate(
			'{className} {',
				'text-align: {alignment};',
			'}',
			'{className} > li > a {',
				'font-size: {level1FontSize}px;',
				'font-family: "{level1FontFamily}";',
				'font-weight: {[values.level1Bold ? "bold" : "normal"]};',
				'font-style: {[values.level1Italic ? "italic" : "normal"]};',
				'text-decoration: {[values.level1Underline ? "underline" : "none"]};',
			'}',
			'{className} > li.selected > a, {className} > li.childselected > a {',
				'font-weight: {[values.level1BoldSelected ? "bold" : "normal"]};',
				'font-style: {[values.level1ItalicSelected ? "italic" : "normal"]};',
			'}',
			'{className} > li.selected > a {',
				'text-decoration: {[values.level1UnderlineSelected ? "underline" : "none"]};',
			'}',
			'{className} > li:hover > a {',
				'font-weight: {[values.level1BoldHovered ? "bold" : "normal"]};',
				'font-style: {[values.level1ItalicHovered ? "italic" : "normal"]};',
				'text-decoration: {[values.level1UnderlineHovered ? "underline" : "none"]};',
			'}',
			'{className} > li.selected:hover > a, {className} > li.childselected:hover > a {',
				'font-weight: {[values.level1BoldSelected ? "bold" : "normal"]};',
			'}',
			'#column1 {className} a, #column2 {className} a { font-size: {level1FontSize}px; /*line-height: {level1LineSize}px;*/ }',
			'',
			'{className} ul > li > a {',
				'font-family: "{level2FontFamily}";',
				'font-size: {level2FontSize}px;',
				'font-weight: {[values.level2Bold ? "bold" : "normal"]};',
				'font-style: {[values.level2Italic ? "italic" : "normal"]};',
				'text-decoration: {[values.level2Underline ? "underline" : "none"]};',
			'}',
			'{className} ul > li:hover > a {',
				'font-weight: {[values.level2BoldHovered ? "bold" : "normal"]};',
				'font-style: {[values.level2ItalicHovered ? "italic" : "normal"]};',
				'text-decoration: {[values.level2UnderlineHovered ? "underline" : "none"]};',
			'}',
			'{className} ul > li.selected > a {',
				'font-weight: {[values.level2BoldSelected ? "bold" : "normal"]};',
				'font-style: {[values.level2ItalicSelected ? "italic" : "normal"]};',
				'text-decoration: {[values.level2UnderlineSelected ? "underline" : "none"]};',
			'}',
			'#column1 {className} ul a, #column2 {className} ul a { font-size: {level2FontSize}px; }'
		);
		Sb.widget.Navigation.cssTemplate.compile();
	}
	return Sb.widget.Navigation.cssTemplate;
};

/**
 * CSS template of navigation widget color scheme.
 * @return {Ext.XTemplate} this
 */
Sb.widget.Navigation.getColorSchemeCssTemplate = function() {
	if (!Sb.widget.Navigation.colorSchemeCssTemplate) {
		Sb.widget.Navigation.colorSchemeCssTemplate = new Ext.XTemplate(
			'<tpl if="level1ContainerBackgroundColor != \'\'">{className} { background-color: {level1ContainerBackgroundColor}; }</tpl>',
			'<tpl if="level1Color != \'\'">{className} > li > a { color: {level1Color};}</tpl>',
			'<tpl if="level1BackgroundColor != \'\'">{className} > li { background-color: {level1BackgroundColor}; }</tpl>',
			'<tpl if="level1ColorSelected != \'\'">{className} > li.selected > a, {className} > li.childselected > a { color: {level1ColorSelected}; }</tpl>',
			'<tpl if="level1BackgroundColorSelected != \'\'">{className} > li.selected, {className} > li.childselected { background-color: {level1BackgroundColorSelected}; }</tpl>',
			'<tpl if="level1ColorHovered != \'\'">{className} > li:hover > a { color: {level1ColorHovered}; }</tpl>',
			'<tpl if="level1BackgroundColorHovered != \'\'">{className} > li:hover { background-color: {level1BackgroundColorHovered}; }</tpl>',
			'<tpl if="level2Color != \'\'">{className} ul > li > a { color: {level2Color}; }</tpl>',
			'<tpl if="level2BackgroundColor != \'\'">{className} ul > li { background-color: {level2BackgroundColor}; }</tpl>',
			'<tpl if="level2ColorSelected != \'\'">{className} ul > li.selected > a { color: {level2ColorSelected}; }</tpl>',
			'<tpl if="level2BackgroundColorSelected != \'\'">{className} ul > li.selected { background-color: {level2BackgroundColorSelected}; }</tpl>',
			'<tpl if="level2ColorHovered != \'\'">{className} ul > li:hover > a { color: {level2ColorHovered}; }</tpl>',
			'<tpl if="level2BackgroundColorHovered != \'\'">{className} ul > li:hover { background-color: {level2BackgroundColorHovered}; }</tpl>'
		);
		Sb.widget.Navigation.colorSchemeCssTemplate.compile();
	}
	return Sb.widget.Navigation.colorSchemeCssTemplate;
};
