/**
 * @class Sb.widget.Text
 * @extends Sb.widget.Widget
 * Class for Text widget
 * @constructor
 * @param {Object} config The config object
 * @xtype text_widget
 */
Sb.widget.Text = Ext.extend(Sb.widget.Widget, {

	cls: 'sb-widget-text sb-widget-text-color',

	hasEditMenu: false,

	afterRender: function() {
		this.update();
		Sb.widget.Text.superclass.afterRender.call(this);
	},

	initEvents: function () {
		Sb.widget.Text.superclass.initEvents.call(this);

		this.model.on('change', this.update, this);
		this.model.on('save', function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(
				SbGetLocaleByKey('messageTextSaved')
			);
		});

		this.mon(this.contentElement, 'contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
			type	: 'widget',
			widget	: this
		});
	},

	update: function() {
		var content = this.model.data.content;
		var pattern = new RegExp('(src="?)(' + Sb.Config.imagePath + '/[^/]+\\.(' + Sb.Config.imageValidExtensions.join('|') + ')"?)', 'img');
		content = content.replace(pattern, '$1' + Sb.Editor.model.getUrl() + '/$2');

		Sb.widget.Text.superclass.update.call(this, content);
	},

	onStartEdit: function() {
		this.memento = this.model.saveToMemento();
		if (!Sb.widget.Text.editPanel) {
			Sb.widget.Text.editPanel = new Sb.widget.TextEditPanel({hidden: true});
		}
		if (!Sb.widget.Text.editPanel.rendered) {
			Sb.widget.Text.editPanel.render(Ext.getBody());
		}
		this.initEditor();
	},

	onCompleteEdit: function() {
		if (!this.instance.initialized) {
			return;
		}

		var content = this.instance.getContent();
		if (content.length > Sb.Config.maxTextWidgetSize) {
			Ext.MessageBox.show({
				msg: SbGetLocaleByKey('textWidgetTooLongContent'),
				buttons: Ext.MessageBox.OK,
				buttonText: {
					ok: SbGetLocaleByKey('ok')
				},
				closable: false
			});
			this.model.unserialize({'content': content});
			return;
		} else if (content.length == 0) {
			this.removeWidgetHandler();
			return;
		}

		var sitesUrl = Sb.Config.sitesUrl;
		var dirName = Sb.Editor.model.data.dirName;
		content		= content.replace(document.location.protocol + '//' + document.location.host, '');
		content		= content.replace(
			new RegExp('(' + sitesUrl + '/' + dirName.substr(0, 2) + '/' + dirName + '/)', 'g'),
			''
		);
		this.model.unserialize({'content': content});
		this.model.saveIfChanged(this.memento);
		var files = Sb.Editor.model.updateUploadedImages().keys;
		if (0 == files.length) {
			files	= ['empty'];
		}
		Sb.WidgetService.removeUnusedUploadedFiles(Sb.Config.imageDirName, files);
	},

	onCancelEdit: function() {
		this.model.restoreFromMemento(this.memento);
	},

	onHideEdit: function() {
		this.removeEditor();
		Sb.widget.Text.editPanel.hide();
	},

	onDocumentClick: function(event) {
		if (event.getTarget('.mceMenu') || event.getTarget('.sb-color-dropdown') || event.getTarget('#mceModalBlocker')
			|| Ext.fly(event.target).up('.mceModal') || event.getTarget('.mceWrapper')
			|| (event.getTarget('.sb-dialog-image')) || Ext.isObject(this.instance.ExpandedSelectbox)) {
			return;
		}
		Sb.widget.Text.superclass.onDocumentClick.call(this, event);
	},

	onDestroy: function() {
		if (this.rendered) {
			this.model.un('change', this.update, this);
		}
		Sb.widget.Text.superclass.onDestroy.call(this);
	},

	initEditor: function(element) {
		var height			= this.el.getSize().height;

		this.instance = new tinymce.Editor(this.contentElement.id, {
			mode		: 'exact',
			height		: height,
			width		: '100%',
			plugins		: 'inlinepopups,media,advlink,tablewithalign,paste',
			body_class	: 'pageContentText sb-widget-text-color',
			theme		: 'sbt',
			language 	: Sb.Config.locale.split('_')[0],

			theme_sbt_resizing_min_height			: 20,
			theme_sbt_toolbar_location				: 'external',
			theme_sbt_toolbar_containerId			: Sb.widget.Text.editPanel.body.id,
			theme_sbt_toolbar_align					: 'left',
			theme_sbt_statusbar_location			: 'none',
			theme_sbt_resizing						: false,
			theme_sbt_buttons1: 'sbformatselect, sbbold, sbitalic, underline, strikethrough,'
								+ 'justifyleft, justifycenter, justifyright, justifyfull, numlist, bullist, media, undo, redo, sbimage',
			theme_sbt_buttons2: 'sbfontselect, sbfontsizeselect, forecolor, backcolor, outdent, indent, table,'
								+ 'hr,charmap, code, sbunlink, link',
			theme_sbt_buttons3: 'row_props, cell_props, row_before, row_after, delete_row, col_before, col_after, delete_col,'
								+ 'split_cells, merge_cells',

			relative_urls							: false,

			setup: function(ed) {
				ed.addButton('sbbold', {
					'class': 'mce_bold',
					cmd: 'empty'
				});
				ed.addButton('sbitalic', {
					'class': 'mce_italic',
					cmd: 'empty'
				});
				ed.addButton('sbformatselect', {
					'class': 'mce_formatselect',
					cmd: 'empty'
				});
				ed.addButton('sbimage', {
					'class': 'mce_image',
					cmd: 'mceImageAction'
				});
				ed.addButton('sbfontselect', {
					'class': 'mce_fontselect',
					cmd: 'empty'
				});
				ed.addButton('sbfontsizeselect', {
					'class': 'mce_fontsize',
					cmd: 'empty'
				});
				ed.addButton('sbunlink', {
					'class': 'mce_unlink',
					title: 'advanced.unlink_desc',
					cmd: 'unlink'
				});
			}
		});

		this.instance.refit = function() {
			var doc, docHeight;
			var frameId = this.id + '_ifr';
			var frame = Ext.get(frameId);
			if (frame) {
				if (!(doc = this.getDoc())) {
					return;
				}
				doc.body.style.overflow = "hidden";
				//Fixes the issue of the table leaving empty space below iframe
				var table = frame.dom.parentNode.parentNode.parentNode.parentNode;
				table.style.height = 'auto';

				if (doc.body.clientHeight) {
					docHeight = doc.body.clientHeight;
				} else {
					docHeight = parseInt(doc.body.scrollHeight);
				}

				var frameHeight = Math.max(docHeight, 10);
				if (frameHeight != frame.getHeight()) {
					frame.setHeight(frameHeight);
					frame.dom.contentWindow.scrollTo(0, 0);
					Sb.widget.Text.editPanel.reanchor();
					// Need to repaint for IE7, otherwise positioning is broken
					if (Ext.isIE7) {
						Sb.widget.Widget.editCurrent.ownerCt.el.repaint();
					}
				}
			}
		};

		this.instance.onPreInit.add(function(ed) {
			Sb.widget.Text.applyCSS(ed);
			ed.controlManager.setDisabled('media', 1);

			p = Ext.fly(ed.controlManager.get('sbfontsizeselect').id).parent();
			Ext.DomHelper.overwrite(p, '');
			new Sb.form.FontSelectBox({
				id: 'mceFontSize',
				renderTo: p,
				fontStoreType: 'size',
				width: 78,
				listeners: {
					scope: this,
					beforeselect: function(combo, record, index) {
						if (combo.getValue() == record.get('value')) {
							combo.collapse();
							return false;
						}
					},
					select: function(combo, record, index) {
						if (Ext.isIE) {
							ed.selection.moveToBookmark(ed.lastSelectionBookmark);
							combo.focusAfterSelectEvent = false;
						}
						ed.execCommand('FontSize', false, record.get('value') + 'px');
					},
					expand: function(combo) {
						ed.ExpandedSelectbox = combo;
					},
					collapse: function(combo) {
						ed.ExpandedSelectbox = null;
					},
					specialkey: function(combo, e) {
						if (e.getKey() == e.ENTER) {
							if (combo.getValue() == combo.getRawValue()) {
								return;
							}
							if (Ext.isIE) {
								ed.selection.moveToBookmark(ed.lastSelectionBookmark);
							}
							if (!combo.isExpanded()) {
								ed.execCommand('FontSize', false, combo.getRawValue() + 'px');
								combo.focus();
							}
						}
					},
					afterrender: function(combo) {
						ed.onNodeChange.add(function(ed, cm, n, co, ob) {
							var fontSizeValue = Ext.fly(n).getStyle('font-size');
							fontSizeValue = (/px$/.test(fontSizeValue)) ? fontSizeValue.replace(/(px)/g, '') : '';
							combo.setValue(fontSizeValue);
						});
					}
				}
			});

			p = Ext.fly(ed.controlManager.get('sbfontselect').id).parent();
			Ext.DomHelper.overwrite(p, '');
			new Sb.form.FontSelectBox({
				id: 'mceFontFamily',
				renderTo: p,
				fontStoreType: 'family',
				width: 112,
				listeners: {
					scope: this,
					beforeselect: function(combo, record, index) {
						if (combo.getValue() == record.get('value')) {
							combo.collapse();
							return false;
						}
					},
					select: function(combo, record, index) {
						if (Ext.isIE) {
							ed.selection.moveToBookmark(ed.lastSelectionBookmark);
							combo.focusAfterSelectEvent = false;
						}
						tinyMCE.execCommand('FontName', false, record.get('value'));
					},
					expand: function(combo) {
						ed.ExpandedSelectbox = combo;
					},
					collapse: function(combo) {
						ed.ExpandedSelectbox = null;
					},
					afterrender: function(combo) {
						ed.onNodeChange.add(function(ed, cm, n, co, ob) {
							combo.setValue(Ext.fly(n).getStyle('font-family').replace(/('|")/g, ''));
						});
					}
				}
			});

			var backColor		= ed.controlManager.get('backcolor'),
				foreColor		= ed.controlManager.get('forecolor'),
				region			= Sb.widget.Widget.editCurrent.ownerCt.model.data.region,
				palette 		= Sb.Editor.model.design.palette,
				backAutoValue	= palette[region + 'Color'],
				foreAutoValue	= palette[region + 'TextColor'];

			backColor.value						= backAutoValue;
			backColor.settings.default_color	= backAutoValue;
			backColor.settings.noColorText		= SbGetLocaleByKey('autoColor');
			foreColor.value						= foreAutoValue;
			foreColor.settings.default_color	= foreAutoValue;
			foreColor.settings.noColorText		= SbGetLocaleByKey('autoColor');
		});

		this.instance.onInit.add(function(ed) {
			ed.onSetContent.add(ed.refit);
			tinyMCE.execCommand('mceFocus', false, ed.editorId);

			Sb.widget.Text.editPanel.toolbar3 = Ext.get(ed.controlManager.prefix + 'toolbar3');
			Sb.widget.Text.editPanel.toolbar3.setVisibilityMode(Ext.Element.DISPLAY);

			ed.addShortcut('ctrl+l', 'left', 'justifyleft');
			Ext.getDom(ed.editorId + '_' + 'justifyleft').title += ' (Ctrl+L)';

			ed.addShortcut('ctrl+e', 'center', 'justifycenter');
			Ext.getDom(ed.editorId + '_' + 'justifycenter').title += ' (Ctrl+E)';

			ed.addShortcut('ctrl+r', 'right', 'justifyright');
			Ext.getDom(ed.editorId + '_' + 'justifyright').title += ' (Ctrl+R)';

			ed.addShortcut('ctrl+j', 'justify', 'justifyfull');
			Ext.getDom(ed.editorId + '_' + 'justifyfull').title += ' (Ctrl+J)';

			p = Ext.fly(ed.controlManager.get('sbimage').id).parent();
			Ext.DomHelper.overwrite(p, '');
			p.addClass('mce-sb-toolbar-button');
			var buttonImg = new Ext.ux.form.FileUploadField({
				id: 'mce-sb-image',
				regex: new RegExp('\\.(' + Sb.Config.imageValidExtensions.join('|') + ')$', 'i'),
				buttonOnly: true,
				name: 'image',
				buttonText: SbGetLocaleByKey('mceInsertImage'),
				listeners: {
					scope: this,
					fileselected: this.onImageUpload,
					afterrender: function(btn) {
						ed.onNodeChange.add(function(ed, cm, n, co, ob) {
							var isImageNode = n.nodeName == 'IMG';
							btn.fileInput[isImageNode ? 'hide' : 'show']();
							btn.button[isImageNode ? 'addClass' : 'removeClass']('x-btn-over');
							btn.button.setText(SbGetLocaleByKey(isImageNode ? 'mceEditImage' : 'mceInsertImage'));
						});
					}
				}
			});
			new Ext.form.FormPanel({
				cls: 'mce-sb-image',
				renderTo: p,
				fileUpload	: true,
				border		: false,
				hideLabels	: true,
				items: buttonImg
			});

			buttonImg.button.on('click', function(){
				var selection = ed.selection.getNode();
				if (selection.nodeName != 'IMG') {
					return;
				}
				ed.imageDlg = new Sb.editor.ImageEditDialog({
					domEl				: ed.selection.getNode(),
					afterHeightChange	: ed.refit
				});
				ed.imageDlg.show();
			});
			buttonImg.button.setIconClass('mce-sb-image-icon');

			buttonImg.button.on('mouseout', function(button, evt){
				if (ed.selection.getNode().nodeName == 'IMG') {
					buttonImg.button.addClass('x-btn-over');
				}
			});

			p = Ext.fly(ed.controlManager.get('link').id).parent();
			Ext.DomHelper.overwrite(p, '');
			p.addClass('mce-sb-toolbar-button');
			new  Ext.Button({
				id: 'id-mce-sb-link',
				cls: 'mce-sb-link',
				iconCls: 'mce-sb-link-icon',
				text: SbGetLocaleByKey('mceInsertLink'),
				renderTo: p,
				disabled: true,
				listeners: {
					scope: this,
					click: function() {
						ed.execCommand('mceAdvLink');
					},
					mouseout: function(btn, evt){
						if (ed.selection.getNode().nodeName == 'A') {
							btn.addClass('x-btn-over');
						}
					},
					afterrender: function(btn) {
						ed.onNodeChange.add(function(ed, cm, n, co, ob) {
							var isLink = n.nodeName == 'A';
							if (co && !isLink) {
								btn.disable();
								btn.setText(SbGetLocaleByKey('mceInsertLink'));
							}
							if (co && isLink) {
								btn.enable();
								btn.setText(SbGetLocaleByKey('mceEditLink'));
							}
							if (ed.selection.getContent()) {
								btn.enable();
								btn.setText(SbGetLocaleByKey('mceInsertLink'));
							}
							btn[isLink ? 'addClass' : 'removeClass']('x-btn-over');
						});
					}
				}
			});

			ed.formatter.register({
				bold : [{inline : 'span', styles : {fontWeight : 'bold'}}],
				italic : [{inline : 'span', styles : {fontStyle : 'italic'}}]
			});
			ed.controlManager.get('sbbold').settings.onclick = function() {
				var fontWeight = Ext.fly(ed.selection.getNode()).getStyle('font-weight');
				var value = (fontWeight == 'bold' || fontWeight == 700) ? 'normal' : 'bold';
				var formatter = ed.formatter.get('bold');
				formatter[0].styles.fontWeight = value;
				ed.formatter.register('bold', formatter);
				ed.execCommand('bold');
			};
			ed.controlManager.get('sbitalic').settings.onclick = function() {
				var value = Ext.fly(ed.selection.getNode()).getStyle('font-style') == 'italic' ? 'normal' : 'italic';
				var formatter = ed.formatter.get('italic');
				formatter[0].styles.fontStyle = value;
				ed.formatter.register('italic', formatter);
				ed.execCommand('italic');
			};

			if (Ext.isSafari) {
				tinymce.dom.Event.add(ed.getBody(), 'dragstart', function(e) {
					if ('IMG' == e.target.nodeName) {
						e.target.ondragend = function (evt) {
							var od = evt.target.ownerDocument;
							od.body.innerHTML = od.body.innerHTML.replace('<img src="' + evt.target.src + '">', evt.target.outerHTML);
						};
					}
				});
			}

			ed.controlManager.get('backcolor').settings.onclick  = ed.controlManager.get('forecolor').settings.onclick = function(v) {
				if (tinymce.isIE) {
					bm = ed.selection.getBookmark(1);
				}
				if (v == null) {
					v = 'transparent';
				}
				ed.execCommand(this.cmd, false, v);
				if (v == 'transparent' || v == this.default_color) {
					ed.formatter.remove(this.cmd.toLowerCase(), {value: v});
				}
			};

			var alignments = {
				alignleft: {'float' : 'left', marginRight : '10px'},
				alignright: {'float' : 'right', marginLeft : '10px'}
			};
			Ext.iterate(alignments, function(alignName, styles) {
				align = ed.formatter.get(alignName);
				align[1].styles = styles;
				ed.formatter.register(alignName, align);
			});

			Sb.widget.Text.editPanel.show(Sb.widget.Widget.editCurrent);

			var p = Ext.fly(ed.controlManager.get('sbformatselect').id).parent();
			Ext.DomHelper.overwrite(p, '');
			new Ext.form.ComboBox({
				store: new Ext.data.ArrayStore({
					fields: ['value', 'display'],
					data: [['p', SbGetLocaleByKey('bodyText')],
						['h1', SbGetLocaleByKey('heading1')],
						['h2', SbGetLocaleByKey('heading2')]
					]
				}),
				triggerAction: 'all',
				mode: 'local',
				valueField: 'value',
				displayField: 'display',
				renderTo: p,
				width: 112,
				tpl: new Ext.XTemplate(
					'<tpl for="."><div class="x-combo-list-item sb-widget-text-color pageContentText">' +
					'<{value} style="background-color: ' + this.ownerCt.el.getStyle('background-color') + '">{display}</{value}>' +
					'</div></tpl>'
				),
				listeners: {
					scope: this,
					expand: function(combo) {
						combo.list.addClass(this.ownerCt.model.data.region);
						combo.list.addClass('mce-sb-formatselect');
						ed.ExpandedSelectbox = combo;
					},
					collapse: function(combo) {
						ed.ExpandedSelectbox = null;
					},
					select: function(combo, record, index) {
						if (Ext.isIE) {
							ed.selection.moveToBookmark(ed.lastSelectionBookmark);
						}
						tinyMCE.execCommand('FormatBlock', false, record.get('value'));
					},
					afterrender: function(combo) {
						ed.onNodeChange.add(function(ed, cm, n, co, ob) {
							for (var i = 0, len = ob.parents.length; i < len; i++) {
								if (tinymce.DOM.isBlock(ob.parents[i])) {
									if (/^(P|H1|H2)$/.test(ob.parents[i].nodeName)) {
										combo.setValue(ob.parents[i].nodeName.toLowerCase());
										return;
									}
									break;
								}
							}
							combo.setValue(SbGetLocaleByKey('customParagraph'));
						});
					}
				}
			});

		}.createDelegate(this));
		this.instance.onGetContent.add(function(ed) {
			document.getElementById(ed.id + '_ifr').setAttribute("allowTransparency", "true");
		});

		this.instance.onNodeChange.add(function(ed, cm, n, co) {
			if (Ext.isObject(ed.ExpandedSelectbox)) {
				ed.ExpandedSelectbox.collapse();
			}
			ed.lastSelectionBookmark = ed.selection.getBookmark(1);
			cm.get('sbunlink').setActive(n.nodeName == 'A');
			cm.get('sbunlink').setDisabled(n.nodeName != 'A');

			var fontWeight = Ext.fly(n).getStyle('font-weight');
			cm.get('sbbold').setActive(fontWeight == 'bold' || fontWeight == 700);
			cm.get('sbitalic').setActive(Ext.fly(n).getStyle('font-style') == 'italic');
		});

		this.instance.onDblClick.add(function(ed, evt) {
			if (ed.selection.getNode().nodeName == 'IMG') {
				ed.imageDlg = new Sb.editor.ImageEditDialog({
					domEl				: ed.selection.getNode(),
					afterHeightChange	: ed.refit.createDelegate(ed)
				});
				ed.imageDlg.show();
			}
		});

		this.instance.onChange.add(this.instance.refit);
		this.instance.onKeyUp.add(this.instance.refit);
		this.instance.onPaste.add(this.instance.refit);

		this.instance.onKeyDown.add(Sb.widget.Text.onKeyDown);

		this.instance.controlsPanel = Sb.widget.Text.editPanel.controlsPanel;
		this.instance.render();
	},

	removeEditor: function() {
		this.instance.remove();
	},

	/**
	 * Image change handler, that fires up when user select image in upload control
	 */
	onImageUpload: function(field, fieldValue) {
		var frm	= field.ownerCt.getForm();
		if (!frm.isValid()) {
			Sb.Editor.Ui.mainWindow.restore();
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(SbGetLocaleByKey('imageEditDialogChangeWarningInvalidType'));
			return;
		}
		frm.submit({
			url			: 'Ajax/Editor/ImageUpload',
			waitMsg		: SbGetLocaleByKey('uploadingImage'),
			success: function(fp, action) {
				var imagePath = Sb.Editor.model.getUrl() + '/' + Sb.Config.imagePath + '/' + encodeURIComponent(action.result.sourceFileName);

				var uniqueId = Ext.id(null, 'mce-');
				var img = '<img id="' + uniqueId + '" src="' + imagePath + '"' +
					'width="'  + action.result.width  + '"' +
					'height="' + action.result.height + '"/>';
				this.instance.execCommand('mceInsertContent', false, img);

				this.instance.imageDlg = new Sb.editor.ImageEditDialog({
					domEl				: this.instance.dom.get(uniqueId),
					afterHeightChange	: this.instance.refit.createDelegate(this.instance),
					isUploaded			: true
				});
				this.instance.imageDlg.show();
				frm.el.dom.reset();
			}.createDelegate(this),
			failure: function(fp, action) {
				Sb.Editor.Ui.mainWindow.restore();
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(SbGetLocaleByKey('imageEditDialogUploadError'));
			}
		});
	}
});

Ext.reg('text_widget', Sb.widget.Text);

Sb.widget.Text.applyCSS = function(ed) {
	var cssPrefix = Sb.Config.templatesUrl + '/' + Sb.Editor.model.data.template.code;
	ed.dom.loadCSS(cssPrefix + '/css/style.css');

	var region = Sb.widget.Widget.editCurrent.ownerCt.model.data.region;
	var head = ed.dom.select('head')[0];
	head.parentNode.className = region;

	Ext.each(['font-set-css', 'text-color-css'], function(id) {
		if (Ext.get(id)) {
			var stylesHTML = Ext.get(id).dom.innerHTML;
			var styles = ed.dom.create('style', {type: 'text/css'});
			if (styles.styleSheet) {// IE
				styles.styleSheet.cssText = stylesHTML;
			} else {// w3c
				styles.appendChild(ed.getDoc().createTextNode(stylesHTML));
			}
			head.appendChild(styles);
		}
	});
};

Sb.widget.Text.onKeyDown = function(inst, e) {
	Sb.app.renewSessionHandler();
	if (e.keyCode) {
		code = e.keyCode;
	} else if (e.which) {
		code = e.which;
	}
	if (code == 9) { //"Tab" pressed
		if (e.shiftKey) {
			inst.execCommand('Outdent');
		} else {
			inst.execCommand('Indent');
		}
		if (e.preventDefault) {
			e.preventDefault();
		}
		return false;
	}
};

Sb.widget.TextEditPanel = Ext.extend(Sb.widget.WidgetEditPanel, {
	initComponent: function() {
		Ext.apply(this, {
			cls: 'sb-widget-edit-panel-text',
			bodyCssClass: 'defaultSkin',
			frame: true
		});

		Sb.widget.TextEditPanel.superclass.initComponent.apply(this, arguments);
	},

	afterRender: function() {
		Sb.widget.Widget.superclass.afterRender.call(this);
		this.moveControl = this.body.createChild({
			cls: 'sb-widget-edit-panel-move',
			html: '<span></span>'
		});
		this.dd = new Ext.dd.DD(this, null, {
			moveOnly: true
		});
		this.dd.editPanel = this;
		this.dd.setHandleElId(this.moveControl);
		this.dd.startDrag = function() {
			this.editPanel.autoAlign = false;
			this.constrainTo(Ext.getBody());
		};
	},

	setToolbarVisibility: function(toolbarName, isVisible) {
		var toolbar = Sb.widget.Text.editPanel[toolbarName];
		if (!isVisible) {
			toolbar.slideOut('t', {
				easing: 'easeOut',
				duration: .3,
				concurrent: false,
				stopFx: true
			});
		} else if (!toolbar.isSlideInActive && (!toolbar.isVisible() || toolbar.hasActiveFx())) {
			toolbar.isSlideInActive = true;
			toolbar.slideIn('t', {
				easing: 'easeIn',
				duration: .3,
				concurrent: false,
				stopFx: true,
				scope: toolbar,
				callback: function() {
					this.isSlideInActive = false;
				}
			});
		}
	}
});

tinymce.Editor.prototype.hide = function() {
	var t = this, d = t.getDoc();

	// Fixed bug where IE has a blinking cursor left from the editor
	if (tinymce.isIE && d) {
		d.execCommand('SelectAll');
	}

	// We must save before we hide so Safari doesn't crash
	// :INFO: echernodarov 20100421: we are not needed in "save" call at "hide" method
	//t.save();
	tinymce.DOM.hide(t.getContainer());
	tinymce.DOM.setStyle(t.id, 'display', t.orgDisplay);
};

tinymce.ui.ColorSplitButton.prototype.setColor = function(c) {
	var t = this;
	t.value = c;

	if (c == null) {
		c = 'transparent';
	}
	tinymce.DOM.setStyle(t.id + '_preview', 'backgroundColor', c);

	t.hideMenu();
	t.settings.onselect(c);
	if (c == 'transparent' || c == t.default_color) {
		tinymce.activeEditor.formatter.remove(t.settings.cmd.toLowerCase(), {value: c});
	}
};

tinymce.ui.ColorSplitButton.prototype.onColorMenuHide = function(menu) {
	tinymce.activeEditor.selection.moveToBookmark(this.selectionBookmark);
	if (!menu.isCancelled) {
		this.setColor(menu.picker.getValue());
	}
};

tinymce.ui.ColorSplitButton.prototype.showMenu = function() {
	this.selectionBookmark = tinymce.activeEditor.selection.getBookmark(1);
	if (!this.menu) {
		this.menu = new Sb.form.ColorMenu({
			value: this.settings.default_color,
			autoValue: this.settings.default_color,
			noColorText: this.settings.noColorText
		});
		this.menu.on('hide', this.onColorMenuHide, this);
	}
	if (this.menu.hidden) {
		this.menu.show(this.id, "tl-bl?");
	}
};

tinymce.ui.ColorSplitButton.prototype.hideMenu = function() {
	if (this.menu && !this.menu.hidden) {
		this.menu.hide();
	}
};

tinymce.ui.ColorSplitButton.prototype.destroy = function() {
	this.parent();

	tinymce.dom.Event.clear(this.id + '_menu');
	tinymce.dom.Event.clear(this.id + '_more');
	tinymce.DOM.remove(this.id + '_menu');

	if (this.menu && !this.menu.isDestroyed) {
		this.menu.destroy();
	}
};