/**
 * @class Sb.widget.Widget
 * @extends Ext.Component
 * Base class for widgets
 * @constructor
 * @param {Object} config The config object
 */
Sb.widget.Widget = Ext.extend(Ext.Component, {
	/**
	 * @cfg {Sb.model.Widget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	/**
	 * @cfg {String} baseCls
	 * The base CSS class to apply to this widget's element (defaults to <code>'sb-widget'</code>).
	 */
	baseCls: 'sb-widget',

	/**
	 * @cfg {Boolean} hasEditMenu
	 * Is needed to create menu for Edit button or not (defaults to <code>true</code>).
	 */
	hasEditMenu: true,

	/**
	 * Returns the <code>id</code> of this widget (sb-widget-UUID)
	 * @return {String} id
	 */
	getId: function() {
		return this.id || (this.id = 'sb-widget-' + this.model.data.uuid);
	},

	onRender: function() {
		this.autoEl = {
			cls: this.baseCls,
			cn: {
				cls: this.baseCls + '-inner',
				cn: [{
					cls: this.baseCls + '-controls',
					cn: {
						cls: this.baseCls + '-controls-wrap',
						cn: {
							cls: this.baseCls + '-controls-inner'
						}
					}
				}, {
					cls: this.baseCls + '-content'
				}]
			}
		};

		Sb.widget.Widget.superclass.onRender.apply(this, arguments);

		this.innerEl = this.el.first();
		this.controlsEl = this.innerEl.first();
		this.controlsEl.setVisibilityMode(Ext.Element.DISPLAY);
		this.contentElement = this.controlsEl.next();
		this.controlsTargetEl = this.controlsEl.first().first();
		this.controlsEl.hide();

		this.ddEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-control-move',
			cn: {
				tag: 'span'
			}
		});

		this.editEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-control-edit'
		});
		this.editEl.button = this.editEl.createChild({
			tag: 'span',
			html: SbGetLocaleByKey('editWidgetTitle')
		});
		this.editEl.arrow = this.editEl.createChild({
			tag: 'i',
			cls: this.baseCls + '-control-dd-arrow'
		});

		this.removeEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-control-remove',
			cn: {
				tag: 'span',
				html: SbGetLocaleByKey('removeWidgetTitle')
			}
		});

		this.applyEl = this.controlsEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-btn ' + this.baseCls + '-control-btn-apply',
			style: 'display: none',
			cn: {
				tag: 'span',
				cls: this.baseCls + '-control-btn-inner',
				cn: {
					tag: 'span',
					html: SbGetLocaleByKey('applyChangesTitle')
				}
			}
		});
		this.applyEl.setVisibilityMode(Ext.Element.DISPLAY).hide();

		this.reverseEl = this.controlsEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-btn ' + this.baseCls + '-control-btn-reverse',
			style: 'display: none',
			cn: {
				tag: 'span',
				cls: this.baseCls + '-control-btn-inner',
				cn: {
					tag: 'span',
					html: SbGetLocaleByKey('reverseChangesTitle')
				}
			}
		});
		this.reverseEl.setVisibilityMode(Ext.Element.DISPLAY).hide();

		if (this.hasEditMenu) {
			this.menu = new Ext.menu.Menu({
				cls: 'sb-menu-widget-actions',
				shadow: false,
				items: [],
				listeners: {
					scope: this,
					mouseover: function(menu, e, menuItem) {
						if (menuItem.tooltip) {
							Sb.Editor.Ui.mainWindow.setHelperMessage(menuItem.tooltip);
						}
					},
					mouseout: function(menu, e, menuItem) {
						if (menuItem.tooltip) {
							Sb.Editor.Ui.mainWindow.restoreHelperMessage();
						}
					},
					show: function(menu) {
						this.editEl.arrow.ignoreNextClick = 0;
						this.editEl.arrow.ignoreNextHide = 0;
						Sb.Editor.Ui.isHoverDisabled = true;
					},
					hide: function(menu) {
						Sb.Editor.Ui.isHoverDisabled = false;
						this.editEl.arrow.ignoreNextClick = this.restoreActionsClick.defer(250, this);
						if (!this.editEl.arrow.ignoreNextHide) {
							this.removeOver.defer(250, this);
						}
					}
				}
			});
		}
	},

	restoreActionsClick: function() {
		this.editEl.arrow.ignoreNextClick = 0;
	},

	restoreActionsMouseDown: function() {
		this.editEl.arrow.ignoreNextHide = 0;
	},

	getContentTarget: function() {
		return this.contentElement;
	},

	afterRender: function() {
		Sb.widget.Widget.superclass.afterRender.call(this);
		this.initEvents();

		if (this.model.isNew) {
			this.model.isNew = false;
			this.el.scrollIntoView(Sb.Editor.siteView.container);
			this.startEdit();
		}
	},

	initEvents: function() {
		Sb.Editor.windowManager.register(this, 1);

		this.contentElement.on('click', function(event) {
			if (this.afterDD) {
				this.afterDD = false;
				return;
			}
			this.startEdit();
		}, this, {stopEvent: true});

		this.getEl().on('mouseenter', function(e) {
			e.stopPropagation();
			if (Sb.widget.Widget.overCurrent == this
				|| Sb.widget.Widget.editCurrent
				|| Sb.Editor.Ui.isDragActive()
				|| Sb.Editor.Ui.isHoverDisabled
			) {
				return;
			}
			this.completeOver();
			Sb.widget.Widget.overCurrent = this;
			if (!this.showOverTask) {
				this.showOverTask = new Ext.util.DelayedTask(this.startOver, this);
			}
			this.showOverTask.delay(200);
		}, this);

		this.getEl().on('mouseleave', this.completeOver, this);

		this.editEl.arrow.on('mousedown', function() {
			this.editEl.arrow.ignoreNextHide = this.restoreActionsMouseDown.defer(250, this);
		}, this);
		this.editEl.arrow.on('click', function() {
			if (!this.editEl.arrow.ignoreNextClick && Sb.widget.Widget.editCurrent != this && this.rendered && this.menu) {
				this.menu.show(this.editEl);
			}
		}, this, {stopEvent:true});
		this.editEl.button.on('click', this.startEdit, this);

		this.removeEl.on('click', this.removeWidget, this, {stopEvent:true});
		this.applyEl.on('click', this.completeEdit, this, {stopEvent:true});
		this.reverseEl.on('click', this.cancelEdit, this, {stopEvent:true});

		this.manager.sendToBack(this);

		this.dd = new Sb.widget.DD(this);
	},

	removeWidget: function() {
		Ext.Msg.show({
			title    : SbGetLocaleByKey('confirmRemoval'),
			msg      : SbGetLocaleByKey('confirmRemovalWidget'),
			closable : false,
			buttons  : Ext.Msg.YESNO,
			scope    : this,
			fn       : function(btn) {
				if (btn == 'yes') {
					this.removeWidgetHandler();
				}
			}
		});
	},

	removeWidgetHandler: function() {
		if (Sb.widget.Widget.editCurrent == this) {
			this.hideEdit();
		}
		this.ownerCt.model.removeChild(this.model);
	},

	showControls: function() {
		this.controlsEl.show();
		this.anchorControls();
	},

	anchorControls: function() {
		this.controlsEl.anchorTo(this.el, 'bl-tl?');
	},

	completeOver: function() {
		if (!Sb.widget.Widget.overCurrent) {
			return;
		}

		if (Sb.widget.Widget.overCurrent.showOverTask) {
			Sb.widget.Widget.overCurrent.showOverTask.cancel();
		}
		Sb.widget.Widget.overCurrent.removeOver();
	},

	startOver: function() {
		if (Sb.widget.Widget.editCurrent) {
			return;
		}
		Sb.widget.Widget.overCurrent = this;
		this.manager.bringToFront(this);
		this.addClass(this.getStateCls('over'));
		this.showControls();
	},

	/**
	 * Starts the editing process and shows the editor.
	 */
	startEdit: function() {
		if (Sb.widget.Widget.editCurrent) {
			if (Sb.widget.Widget.editCurrent == this) {
				return;
			}
			Sb.widget.Widget.editCurrent.completeEdit.call(Sb.widget.Widget.editCurrent);
		}
		if (Sb.widget.Widget.overCurrent) {
			Sb.widget.Widget.overCurrent.removeOver.call(Sb.widget.Widget.overCurrent);
		}
		Sb.widget.Widget.editCurrent = this;
		this.manager.bringToFront(this);
		this.applyEl.show();
		this.reverseEl.show();
		this.showControls();
		this.editEl.addClass(this.baseCls + '-control-item-disabled');
		this.el.addClass(this.getStateCls('focus'));
		this.onStartEdit();
		Ext.getDoc().on('mousedown', this.onDocumentClick, this);
	},

	onStartEdit: Ext.emptyFn,

	/**
	 * Ends the editing process, persists the changed value to the underlying field, and hides the editor.
	 */
	completeEdit: function() {
		if (Sb.widget.Widget.editCurrent != this) {
			return;
		}
		this.onCompleteEdit();
		this.hideEdit();
	},

	onCompleteEdit: Ext.emptyFn,

	/**
	 * Cancels the editing process and hides the editor without persisting any changes.
	 */
	cancelEdit: function() {
		if (Sb.widget.Widget.editCurrent != this) {
			return;
		}
		this.onCancelEdit();
		this.hideEdit();
	},

	onCancelEdit: Ext.emptyFn,

	/**
	 * Hides the editor.
	 */
	hideEdit: function() {
		Ext.getDoc().un('mousedown', this.onDocumentClick, this);
		this.onHideEdit();
		this.applyEl.hide();
		this.reverseEl.hide();
		this.controlsEl.hide();
		this.editEl.removeClass(this.baseCls + '-control-item-disabled');
		this.el.removeClass(this.getStateCls('focus'));
		this.el.removeClass(this.getStateCls('over'));
		Sb.widget.Widget.editCurrent = null;
	},

	onHideEdit: Ext.emptyFn,

	/**
	 * Remove "over" state
	 */
	removeOver: function() {
		if ((!Ext.dd.DragDropMgr.dragCurrent || Ext.dd.DragDropMgr.dragCurrent.widget != this)
				&& Sb.widget.Widget.editCurrent != this && (!this.menu || this.menu.hidden)) {
			Sb.widget.Widget.overCurrent = null;
			this.manager.sendToBack(this);
			this.controlsEl.hide();
			this.removeClass(this.getStateCls('over'));
		}
	},

	/**
	 * Tries to get edit panel for current type or it`s parent.
	 *
	 * @param {xtype} type
	 * @return {Sb.widget.WidgetEditPanel} editPanel
	 */
	getEditPanel: function(type) {
		if (undefined == type) {
			type = Ext.ComponentMgr.types[Sb.widget.Widget.editCurrent.xtype];
		}
		if (type.editPanel) {
			return type.editPanel;
		} else {
			return this.getEditPanel(type.superclass.constructor);
		}
	},

	onDocumentClick: function(event) {
		var editPanel = this.getEditPanel();
		if (event.within(editPanel.el)
			|| event.within(this.el)
			|| (this.menu && event.within(this.menu.el))
			|| event.target == Sb.Editor.Ui.mainWindow.header.dom
			|| event.getTarget('#sb-editor-minimize-btn')
			|| event.getTarget('#sb-editor-help-btn')
			|| Ext.fly(event.target).hasClass('ext-el-mask')
			|| event.getTarget('.x-window-dlg')
			|| (Ext.fly(event.target).hasClass('sb-frame') && ((event.xy[0] >= event.target.clientWidth) || (event.xy[1] >= event.target.clientHeight)))
			|| event.button != 0
			|| (Ext.isSafari && (
					event.xy[0] > event.target.clientWidth
					|| event.xy[1] > event.target.clientHeight
					|| event.xy[0] < 0
					|| event.xy[1] < 0
				)
			)
		) {
			return;
		}

		this.completeEdit();
	},

	/**
	 * Gets class for some state
	 *
	 * @param {String} state
	 * @return {String} class
	 **/
	getStateCls: function(state) {
		var cls = this.baseCls;
		if (this.model instanceof Sb.model.SharedWidget) {
			cls += '-shared';
		}
		cls += '-' + state;
		return cls;
	},

	resetStyles: function() {
		this.getEl().setStyle({
			position	: 'relative',
			left		: 'auto',
			top			: 'auto',
			width		: '100%'
		});
		this.removeClass(this.getStateCls('over'));
	},

	// support manager interface
	setZIndex: function(zindex) {
		this.getEl().setStyle('z-index', zindex);
	},

	// support manager interface
	setActive: Ext.emptyFn,

	beforeDestroy: function() {
		Sb.widget.Widget.superclass.beforeDestroy.call(this);
		if (Sb.widget.Widget.editCurrent == this) {
			this.hideEdit();
		}
		if (this.menu) {
			Ext.destroy(this.menu);
		}
		if (this.controlsEl) {
			this.controlsEl.removeAnchor();
		}
	},

	onDestroy: function() {
		Sb.widget.Widget.superclass.onDestroy.call(this);
		if (this.rendered) {
			this.model.destroy();
		}
		this.manager.unregister(this);
		if (Sb.widget.Widget.overCurrent == this) {
			if (Sb.widget.Widget.overCurrent.showOverTask) {
				Sb.widget.Widget.overCurrent.showOverTask.cancel();
			}
			Sb.widget.Widget.overCurrent = null;
		}
		if (Sb.widget.Widget.editCurrent == this) {
			Sb.widget.Widget.editCurrent = null;
		}
	}
});

//private - custom Widget DD implementation
Sb.widget.DD = function(widget, cfg) {
	this.widget = widget;
	cfg = Ext.apply({ddGroup: 'widgets'}, cfg);

	Sb.widget.DD.superclass.constructor.call(this, this.widget.el, cfg);
	this.setOuterHandleElId(this.widget.ddEl);
	this.widget.ddEl.setStyle('cursor', 'move');
	this.scroll = false;
};

Ext.extend(Sb.widget.DD, Ext.dd.DragSource, {
	getDragEl: function() {
		return Ext.getDom(this.widget.getEl());
	},

	b4MouseDown: function(e) {
		var x = e.getPageX();
		var y = e.getPageY();
		this.autoOffset(x, y);
	},

	autoOffset: function(x, y) {
		x -= this.startPageX;
		y -= this.startPageY;
		this.setDelta(x, y);
	},

	startDrag: function (x, y) {
		Sb.Editor.siteView.onWidgetStartDrag();

		if (Sb.widget.Widget.editCurrent) {
			Sb.widget.Widget.editCurrent.completeEdit.call(Sb.widget.Widget.editCurrent);
			if (!this.widget.controlsEl.isVisible()) {
				this.widget.showControls();
			}
		}

		// :TODO: echernodarov 20100420: dirty hack, need rework
		this.widget.afterDD = true;
		this.widget.setZIndex(Sb.Editor.windowManager.dragZIndex);
		this.widget.ownerCt.notifyOver(this, [x, y]);
		this.widget.el.setStyle({
			position: 'absolute',
			width: this.widget.el.getWidth() + 'px'
		});
		if (Ext.isIE7) {

			/**
			 * Widget grand-parent container during DD
			 *
			 * @type Ext.Element
			 * @property ddWidgetContainer
			 */
			this.ddWidgetContainer = this.getWidgetContainer();
			if (this.ddWidgetContainer) {
				this.ddWidgetContainer.setStyle('z-index', Sb.Editor.windowManager.dragZIndex);
			}
		}
	},

	getWidgetContainer: function() {
		return this.widget.el.up('.container-content-inner', 1).up('div', 1);
	},

	endDrag: function(e) {
		if (Ext.isIE7) {
			if (this.ddWidgetContainer) {
				this.ddWidgetContainer.setStyle('z-index', 0);
			}
			this.ddWidgetContainer = null;
		}
		delete this.invalidDrop;
		Sb.Editor.siteView.onWidgetEndDrag();
	},

	onInvalidDrop: function () {
		this.invalidDrop = true;
		var animCfgObj = {
			easing		: 'elasticOut',
			duration	: 0.3,
			scope		: this,
			callback	: function() {
				Sb.Editor.siteView.ddSpacer.remove();
				this.widget.resetStyles();
			}
		};

		this.el.moveTo(this.startPageX, this.startPageY, animCfgObj);
	},
	createFrame	: Ext.emptyFn,
	showFrame	: Ext.emptyFn
});

Sb.widget.WidgetEditPanel = Ext.extend(Ext.Panel, {
	initComponent: function() {
		Ext.apply(this, {
			autoRender: true,
			baseCls: 'sb-widget-edit-panel',
			layout: 'fit'
		});

		Sb.widget.WidgetEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function(widget) {
		this.widget = widget;
		Sb.widget.WidgetEditPanel.superclass.show.call(this);
		this.autoAlign = true;
		this.reanchor();
		this.el.setStyle('z-index', parseInt(widget.el.getStyle('z-index')) + 1);
	},

	reanchor: function() {
		this.widget.anchorControls();
		if (this.autoAlign) {
			this.alignTo(this.widget.el, 'tl-bl?', [0, 5], true);
		}
	},

	alignTo : function(element, position, offsets, animate) {
		var me = this.el, maxY;
		if (this.widget.controlsEl.getY() > this.widget.el.getY()) {
			offsets[1] += this.widget.controlsEl.getHeight();
		}
		var xy = me.getAlignToXY(element, position, offsets);
		if (xy[1] < this.widget.el.getY()) {
			xy[1] = Math.max(xy[1] - this.widget.controlsEl.getHeight(), 0);
		}
		if (xy[1] > (maxY = Ext.lib.Dom.getViewHeight() - me.getHeight())) {
			xy[1] = maxY;
		}
		return me.setXY(xy,
			me.preanim && !!animate ? me.preanim(arguments, 3) : false);
	}
});