/**
 * @class Sb.widgetType.Base
 */
Sb.widgetType.Base = Ext.extend(Object, {

	/**
	 * Name of the type
	 *
	 * @type String
	 */
	name: '',

	constructor: function(name) {
		if (name) {
			this.name = name;
		}

		/**
		 * Components, that is registered for each type
		 *
		 * @type Ext.util.MixedCollection
		 * @property components
		 */
		this.components = new Ext.util.MixedCollection();

		this.components.on('add', this.onComponentsAdd, this);
	},

	/**
	 * Automatic convertion to string for simple use cases
	 * @return {String}
	 */
	toString: function() {
		return this.name;
	},

	/**
	 * Gets type for creating child
	 * @return {String}
	 */
	getPhpType: function() {
		return this.name;
	},

	/**
	 * Get key for collections
	 * @return {String}
	 */
	getKey: function() {
		return this.name;
	},

	/**
	 * Get title for presentation elements
	 * @return {String}
	 */
	getTitle: function() {
		return SbGetLocaleByKey(this.name);
	},

	/**
	 * Get description for presentation elements
	 * @param {Boolean} disabled
	 * @return {String}
	 */
	getDescription: function(disabled) {
		var key = this.name + 'Description' + (disabled ? 'Disabled' : '');
		return SbGetLocaleByKey(key);
	},

	/**
	 * Toggle all registered components to state
	 * @param {Boolean} state
	 */
	toggle: function(state) {
		this.components.each(function (cmp) {
			state ? cmp.enable() : cmp.disable();
		});
	},

	/**
	 * Enables all registered components
	 */
	enable: function() {
		this.toggle(true);
	},

	/**
	 * Disable all registered components
	 */
	disable: function() {
		this.toggle(false);
	},

	/**
	 * Determine whether we can drop widget to region
	 *
	 * @param {String} region
	 */
	isDroppableTo: function(region) {
		return true;
	},

	/**
	 * Determine whether we can create new instances
	 */
	isAvailable: function() {
		return true;
	},

	/**
	 * Instance factory
	 *
	 * @param {Object} config The configuration options for the widget model
	 * @return {Sb.model.Widget}
	 */
	createInstance: function(config) {
		var type = config.type.camelize(true) + 'Widget';
		if (Sb.model[type]) {
			return new Sb.model[type](config);
		}
		return new Sb.model.Widget(config);
	},

	/**
	 * Widget cleaner by type
	 *
	 * @param {Sb.model.Widget} widget
	 */
	removeInstance: Ext.emptyFn,

	/**
	 * Restore widget in cache
	 *
	 * @param {Sb.model.Widget} widget
	 */
	restoreInstance: Ext.emptyFn,

	/**
	 * Fires when component register
	 */
	onComponentsAdd: Ext.emptyFn
});
